/** @file cloneable.hh
    Declares interface to make a class cloneable.
    @author Philipp Frauenfelder, 2004
*/

#ifndef cloneable_hh
#define cloneable_hh

namespace concepts {

  /** Cloneable interface.
      A class inheriting from Cloneable must implement clone and has then
      a virtual constructor (cf. Stroustrup p. 424).
      @author Philipp Frauenfelder, 2004
  */
  class Cloneable {
  public:
    /** Virtual constructor. Returns a pointer to a copy of itself. The caller
	is responsible to destroy this copy.
    */
    virtual Cloneable* clone() const = 0;
  };

} // namespace concepts

#endif // cloneable_hh
