/* level handling
 */

#include "level.hh"

namespace concepts {

  // ***************************************************************** Level **

  template<uint dim>
  Level<dim>& Level<dim>::operator=(const Level<dim>& c) {
    if (&c != this) {
      for (uint i = 0; i < dim; ++i) l_[i] = c.l_[i];
    }
    return *this;
  }

  template<uint dim>
  bool Level<dim>::operator<(const Level<dim>& c) const {
    for (uint i = 0; i < dim; ++i)
      if (l_[i] < c.l_[i]) return true;
    return false;
  }

  template<uint dim>
  bool Level<dim>::operator>(const Level<dim>& c) const {
    for (uint i = 0; i < dim; ++i)
      if (l_[i] > c.l_[i]) return true;
    return false;
  }

  template<uint dim>
  bool Level<dim>::operator==(const Level<dim>& c) const {
    bool res = true;
    for (uint i = 0; i < dim; ++i) res &= (l_[i] == c.l_[i]);
    return res;
  }

  template<uint dim>
  Level<dim> Level<dim>::operator+(const Level<dim>& c) const {
    Level<dim> res(*this);
    for (uint i = 0; i < dim; ++i) res.l_[i] += c.l_[i];
    return res;
  }

  template<uint dim>
  Level<dim> Level<dim>::operator-(const Level<dim>& c) const {
    Level<dim> res(*this);
    for (uint i = 0; i < dim; ++i) res.l_[i] -= c.l_[i];
    return res;
  }

  template<uint dim>
  void Level<dim>::minimize(const Level<dim>& c) {
    for (uint i = 0; i < dim; ++i)
      if (l_[i] > c.l_[i]) l_[i] = c.l_[i];
  }

  template<uint dim>
  void Level<dim>::maximize(const Level<dim>& c) {
    for (uint i = 0; i < dim; ++i)
      if (l_[i] < c.l_[i]) l_[i] = c.l_[i];
  }

  template<uint dim>
  std::ostream& operator<<(std::ostream& os, const Level<dim>& c) {
    os << "Level<" << dim << ">(";
    for (uint i = 0; i < dim-1; ++i) os << c.l_[i] << ", ";
    return os << c.l_[dim-1] << ')';
  }

  template class Level<1>;
  template class Level<2>;
  template class Level<3>;

  template std::ostream& operator<<(std::ostream&, const Level<1>&);
  template std::ostream& operator<<(std::ostream&, const Level<2>&);
  template std::ostream& operator<<(std::ostream&, const Level<3>&);

} // namespace concepts
