// abstract class for a eigensolver fabric

#ifndef solverFabric_hh
#define solverFabric_hh

#include "basics/outputOperator.hh"

namespace concepts {

  // forward declaration

  template<typename F>
  class Operator;

}

namespace eigensolver {

  // ********************************************************** SolverFabric **

  /** Abstract fabric class for eigenvalue solvers. This class is intended
      to be used for constrained problems, ie. as input to Constrained.
      @author Philipp Frauenfelder, 2002
  */
  class SolverFabric : public concepts::OutputOperator {
  public:
    /** Creates an eigenvalue solver which solves the abstract eigenvalue
	problem defined by \c stiff and \c mass. This fabric should only
	by used once as subsequent uses would invalidate older solvers.
	@return Pointer to eigenvalue solver
    */
    virtual EigenSolver& operator()(concepts::Operator<Real>& stiff,
				    concepts::Operator<Real>& mass) = 0;
  };

} // eigensolver

#endif // solverFabric_hh
