/* testsuite for Vector
 */

#include <cstdlib>
#include "basics/exceptions.hh"
#include "function/vector.hh"
#include "vectorTest.hh"

using concepts::Real;
using concepts::Cmplx;

namespace test {

  void VectorTest::run() {
    _test(spc.dim() == 10);
    concepts::Vector<Real> v(spc);
    for (uint i = 0; i < spc.dim(); ++i)
      v[i] = 1.+i;
    testl1norm(v); testl2norm(v); testmax(v); testreadwrite(v);

    _test(spcC.dim() == 10);
    concepts::Vector<Cmplx> vC(spcC);
    for (uint i = 0; i < spcC.dim(); ++i)
      vC[i] = Cmplx(1.,i);
    testl1normC(vC); testl2normC(vC); testmaxC(vC); testreadwriteC(vC);
  }

  void VectorTest::testl1norm(const concepts::Vector<Real>& v) {
    _numtest(v.l1(), 55);
  }

  void VectorTest::testl2norm(const concepts::Vector<Real>& v) {
    _numtest(v.l2(), 19.621416870348582506);
  }

  void VectorTest::testmax(const concepts::Vector<Real>& v) {
    _numtest(v.max(), 10);
  }

  void VectorTest::testreadwrite(const concepts::Vector<Real>& v) {
    v.write("vector.testwrite");
    concepts::Vector<Real> v2(spc, "vector.testwrite");
    _test(v.space().dim() == v2.space().dim());
    for (uint i = 0; i < v.space().dim(); ++i) {
      _numtest(v[i], v2[i]);
      _numtest(v2[i], 1.+i);
    }

    concepts::Vector<Real> v3(spc, SOURCEDIR
			      "/function/testsuite/vector.test");
    _test(v.space().dim() == v3.space().dim());
    for (uint i = 0; i < v.space().dim(); ++i) {
      _numtest(v[i], v3[i]);
    }
    _test(0 == std::system("diff vector.testwrite "
			   SOURCEDIR "/function/testsuite/vector.test"));
  }

  void VectorTest::testl1normC(const concepts::Vector<Cmplx>& v) {
    _numtest(v.l1(), 47.306157567851371937);
  }

  void VectorTest::testl2normC(const concepts::Vector<Cmplx>& v) {
    _numtest(v.l2(), 17.175564037317666788);
  }

  void VectorTest::testmaxC(const concepts::Vector<Cmplx>& v) {
    _numtest(v.max(), 9.0553851381374173002);
  }

  void VectorTest::testreadwriteC(const concepts::Vector<Cmplx>& v) {
    v.write("vectorC.testwrite");
    concepts::Vector<Cmplx> v2(spcC, "vectorC.testwrite");
    _test(v.space().dim() == v2.space().dim());
    for (uint i = 0; i < v.space().dim(); ++i) {
      _numtest(v[i].real(), v2[i].real());
      _numtest(v[i].imag(), v2[i].imag());
      _numtest(v2[i].real(), 1.);
      _numtest(v2[i].imag(), i);
    }

    concepts::Vector<Cmplx> v3(spcC, SOURCEDIR
			      "/function/testsuite/vectorC.test");
    _test(v.space().dim() == v3.space().dim());
    for (uint i = 0; i < v.space().dim(); ++i) {
      _numtest(v[i].real(), v3[i].real());
      _numtest(v[i].imag(), v3[i].imag());
    }
    _test(0 == std::system("diff vectorC.testwrite "
			   SOURCEDIR "/function/testsuite/vectorC.test"));
  }

} // namespace test

int main(int, char**) {
  try {
    test::VectorTest t;
    t.run();
    return t.report();
  }
  catch (concepts::ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }
}
