/** @file vectorTest.hh
    Testsuite for concepts::Vector
    @author Philipp Frauenfelder, 2004
*/

#ifndef vectorTest_hh
#define vectorTest_hh

#include "basics/testcase.hh"
#include "space/space.hh"

namespace concepts {
  template<typename F>
  class Vector;
}

namespace test {

  /** Test Vector.
      @author Philipp Frauenfelder, 2004
      @see concepts::Vector
  */
  class VectorTest : public TestCase {
  public:
    VectorTest() : spc(10), spcC(10) {}
    virtual ~VectorTest() {}
    virtual void run();

    /// @name Test routines
    //@{
    /// Tests l1 norm
    void testl1norm(const concepts::Vector<concepts::Real>& v);
    void testl1normC(const concepts::Vector<concepts::Cmplx>& v);
    /// Tests l2 norm
    void testl2norm(const concepts::Vector<concepts::Real>& v);
    void testl2normC(const concepts::Vector<concepts::Cmplx>& v);
    /// Tests linfty norm
    void testmax(const concepts::Vector<concepts::Real>& v);
    void testmaxC(const concepts::Vector<concepts::Cmplx>& v);
    /// Tests read and write operations (with files)
    void testreadwrite(const concepts::Vector<concepts::Real>& v);
    void testreadwriteC(const concepts::Vector<concepts::Cmplx>& v);
    //@}
  private:
    concepts::DummySpace<concepts::Real> spc;
    concepts::DummySpace<concepts::Cmplx> spcC;
  };

} // namespace test

#endif // vectorTest_hh
