/* Interface to create boundary conditions
 */

#ifndef semantics_hh
#define semantics_hh

#include "basics/outputOperator.hh"

namespace concepts {

  // forward declarations
  class Attribute;

  // ************************************************************* Semantics **

  /** An abstract function class to query attributes.
   */
  template<class T>
  class Semantics : public OutputOperator {
  public:
    /// Application operator to query an attribute
    virtual T operator()(Attribute attrib) const = 0;
  protected:
    virtual std::ostream& info(std::ostream& os) const {
      return os << "Semantics()"; }
  };

  // ********************************************************* AttributeBool **

  /** A function class to query attributes. Its application operator
      returns true if the attribute is nonzero.
  */
  class AttributeBool : public Semantics<bool> {
  public:
    /** Application operator.
	@return True, if attrib is non-zero and false if attrib is zero.
    */
    bool operator()(Attribute attrib) const;
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  };

} // namespace concepts

#endif // semantics_hh
