/* testsuite for 3D cells with geometric deadlock
 * subdivide cells
 */

#include <memory>
#include "basics/exceptions.hh"
#include "geometry.hh"
#include "deadlock1.hh"

namespace test {

  // ************************************************************* DeadLock1 **

  void DeadLock1::run() {
    testSubdivs();
  }

  void DeadLock1::testSubdivs() {
    using namespace concepts;

    try {
      Import3dMesh mesh
	(SOURCEDIR "/geometry/testsuite/deadlock1.Coord.dat",
	 SOURCEDIR "/geometry/testsuite/deadlock1.Elements.dat",
	 SOURCEDIR "/geometry/testsuite/deadlock1.Boundary.dat");
      _test(mesh.ncell() == 2);

      const Hex3dSubdivision* subdivs[] = {
	Hex3dSubdiv2z::instance(), Hex3dSubdiv4y::instance(),
	Hex3dSubdiv2x::instance()
      };

      const uint faceIdx[] = { 1, 2, 3, 0, 4, 5, 6, 0, 7, 8, 9, 10 };
      const uint childIdx[] = { 3, 7 };
      const uint grandchildIdx[] = { 4, 10 };

      concepts::Stacktrace::doit() = false;

      std::auto_ptr<Scan3> sc(mesh.scan());
      uint i = 0;
      while (*sc) {
	Cell3& cell = (*sc)++;
	_test(cell.connector().key().key() == i);
	const Connector3& cntr3 = cell.connector();
	for (uint j = 0; j < 6; ++j)
	  _test(cntr3.face(j)->key().key() == faceIdx[j + 6*i]);
	Hexahedron3d* ch = dynamic_cast<Hexahedron3d*>(&cell);
	if (ch) {
	  ch->setStrategy(subdivs[i]);
	  _test(ch->getStrategy() == subdivs[i]);
	  _test(cell.child(1)->connector().key().key() == childIdx[i]);
	  ch->child(1)->setStrategy(subdivs[2]);
	  _test(cell.child(1)->child(0)->connector().key().key() ==
		grandchildIdx[i]);
	}
	++i;
      } // scanner

    } // try
    catch (ExceptionBase& e) {
      std::cout << e << std::endl;
      _fail("caught exception");
    }
  }

} // namespace test

// ****************************************************************** main() **

int main(int, char**) {
  test::DeadLock1 t;
  t.run();
  return t.report();
}
