/* Test suite for mesh converter
 */

#include "geometry/meshConverter.hh"
#include "geometry/meshImport2D.hh"
#include "meshConverter.hh"

namespace test {

  // ****************************************************** ConvertMeshQuads **

  void ConvertMeshQuads::run() {
    testEmpty(); testQuadMesh(); testTriangleMesh(); testMixedMesh();
  }

  void ConvertMeshQuads::testEmpty() {
    concepts::Import2dMesh m(SOURCEDIR "/geometry/testsuite/empty.dat",
			     SOURCEDIR "/geometry/testsuite/empty.dat",
			     SOURCEDIR "/geometry/testsuite/empty.dat");
    _test(m.ncell() == 0);
    concepts::ConvertMeshQuads c(m, 0);
    _test(c.ncell() == 0);
  }

  void ConvertMeshQuads::testQuadMesh() {
    concepts::Import2dMesh m
      (SOURCEDIR "/geometry/testsuite/Coordinates2D.dat",
       SOURCEDIR "/geometry/testsuite/quad.dat",
       SOURCEDIR "/geometry/testsuite/empty.dat");
    _test(m.ncell() == 1);
    concepts::ConvertMeshQuads c(m, 0);
    _test(c.ncell() == 1);

    concepts::ConvertMeshQuads c2(m, 1);
    _test(c2.ncell() == 4);
  }

  void ConvertMeshQuads::testTriangleMesh() {
    concepts::Import2dMesh m
      (SOURCEDIR "/geometry/testsuite/Coordinates2D.dat",
       SOURCEDIR "/geometry/testsuite/Elements2D.dat",
       SOURCEDIR "/geometry/testsuite/empty.dat");
    _test(m.ncell() == 1);
    concepts::ConvertMeshQuads c(m, 0);
    _test(c.ncell() == 3);

    concepts::ConvertMeshQuads c2(m, 1);
    _test(c2.ncell() == 12);
  }

  void ConvertMeshQuads::testMixedMesh() {
    concepts::Import2dMesh m
      (SOURCEDIR "/geometry/testsuite/Coordinates2D.dat",
       SOURCEDIR "/geometry/testsuite/mixed.dat",
       SOURCEDIR "/geometry/testsuite/empty.dat");
    _test(m.ncell() == 2);
    try {
      concepts::ConvertMeshQuads c(m, 0);
      _fail("should have thrown an exception");
    }
    catch (concepts::MissingFeature& e) {
      _succeed();
    }
  }

} // namespace test

// ****************************************************************** main() **

int main(int, char**) {
  try {
    concepts::Stacktrace::doit() = false;
    test::ConvertMeshQuads t;
    t.run();
    return t.report();
  }
  catch (concepts::ExceptionBase& e) {
    std::cout << e << std::endl;
  }
  return 1;
}
