/* Topology.
 * how to subdivide
 */

#include <sstream>
#include "topology.hh"
#include "basics/debug.hh"

// debugging Edge
#define EdgeConstr_D 0

// debugging Triangle
#define TriangleChild_D 0

// debuggin Quad
#define QuadConstr_D 0
#define QuadDestr_D 0
#define QuadsetStrategy_D 0
#define QuadChild_D 0

namespace concepts {

  // ***************************************************************** match **

  int match(const Connector1& edg0, const Connector1& edg1, int idx[]) {

    for (uint i = 0; i < 2; ++i)
      for (uint j = 0; j < 2; ++j)
	if (edg0.vertex(i) == edg1.vertex(j)) {
	  idx[0] = i;
	  idx[1] = j;
	  return 1;
	}
    return 0;
  }

  // ************************************************************ commonEdge **

  // function for finding a common edge of two cells
  // its assumed, the cell has at most 4 edges
  int commonEdge(const Connector2& cell0, const Connector2& cell1, int idx[]) {
    for (uint i = 0; i < 4; ++i)
      if (cell0.edge(i)) {
	for (uint j = 0; j < 4; ++j)
	  if (cell1.edge(j))
	    if (cell0.edge(i) == cell1.edge(j)) {
	      idx[0] = i;
	      idx[1] = j;
	      return 1;
	    }
	}
    return 0;
  }

  // **************************************************************** Vertex **

  Vertex::~Vertex() {}

  std::ostream& Vertex::info(std::ostream& os) const {
    return os << "Vertex(" << key_
// 	      << ", " << attrib_
	      << ')';
  }

  // ****************************************************************** Edge **

  Edge::Edge(Vertex& vtx0, Vertex& vtx1, const Attribute attrib) :
    Connector1(attrib), lnk_(0), chld_(0), regular_(true) {
    vtx_[0] = &vtx0; vtx_[1] = &vtx1;
  }


  // the non-regular boolean is needed at least for 
  // non-deactivating edges in buildTColumns_
  Edge::Edge(Edge& edg0, Edge& edg1) : 
    Connector1(edg0.attrib()), chld_(0), regular_(false)
  {
    DEBUGL(EdgeConstr_D, "Edge 0 = " << edg0);
    DEBUGL(EdgeConstr_D, "Edge 1 = " << edg1);

    // same atrribute
    conceptsAssert(edg0.attrib() == edg1.attrib(), Assertion());

    // common vertex
    int i[2];
    conceptsAssert(match(edg0, edg1, i), Assertion());

    vtx_[0] = edg0.vertex((i[0]+1)%2);
    vtx_[1] = edg1.vertex((i[1]+1)%2);

    DEBUGL(EdgeConstr_D, key() << " - done.");
  }

  Edge::~Edge() {

    if ((regular_) && (chld_ != 0)) {
      delete chld_->vtx_[1];
      delete chld_->lnk_;
      delete chld_;
    }
  }

  Edge* Edge::child(uint i, bool mode) {
    if ((i > 1) || (!regular_)) return 0;

    if (mode)  return i ? (chld_ ? chld_->lnk_ : 0) : chld_;

    if (chld_ == 0) {
      Vertex* vtx = new Vertex(attrib_);
      chld_ = new Edge(*vtx_[0]->child(0), *vtx, attrib_);
      chld_->lnk_ = new Edge(*vtx, *vtx_[1]->child(0), attrib_);
    }

    return i ? chld_->lnk_ : chld_;
  }

  const Edge* Edge::child(uint i) const {
    if ((i > 1) || (!regular_)) return 0;
    return i ? (chld_ ? chld_->lnk_ : 0) : chld_;
  }

  Edge::Edge(Vertex& vtx0, Vertex& vtx1, const Attribute attrib,
	     const Key& key) : Connector1(attrib, key), lnk_(0), chld_(0) {
    vtx_[0] = &vtx0;  vtx_[1] = &vtx1;
  }

  std::ostream& Edge::info(std::ostream& os) const {
    return os << "Edge(" << key_ << ", (" << *vtx_[0] << ", "
	      << *vtx_[1] << "), " << attrib_
// 	      << ", " << ((chld_ == 0) ? "no" : "has") << " children"
	      << ')';
  }

  // ************************************************************** Triangle **

  Triangle::Triangle(Edge& edg0, Edge& edg1, Edge& edg2,
		     const Attribute attrib) : 
    Connector2(attrib), lnk_(0), chld_(0) {

    edg_[0] = &edg0; edg_[1] = &edg1; edg_[2] = &edg2;

    int i[2];
    match(edg0, edg1, i); 

    rho_[0] = i[0] + 1;
    rho_[1] = i[1];
    rho_[2] = (edg0.vertex(rho_[0]) == edg2.vertex(0)) ? 1 : 0;
  }

  Triangle::~Triangle() {
  
    if (chld_ != NULL) {
      Triangle* tri = chld_; chld_ = chld_->lnk_;
      delete tri->edg_[1];
      delete tri;

      tri = chld_; chld_ = chld_->lnk_;
      delete tri->edg_[2];
      delete tri;

      tri = chld_; chld_ = chld_->lnk_;
      delete tri->edg_[0];
      delete tri;

      delete chld_;
    }
  }

  Triangle* Triangle::child(uint i, bool mode) {

    DEBUGL(TriangleChild_D, key_ << " -- i = " << i);
    if (i > 3) return 0;

    if (mode) {
      if (chld_) {
	Triangle* t = chld_;
	while (i--) t = t->lnk_;
	return t;
      }
      return 0;
    }

    if (chld_ == NULL) {
      DEBUGL(TriangleChild_D, key_ << " -- subdiving " << *this);

      Edge* edgA = edg_[0]->child(rho_[0]);
      Edge* edgB = edg_[0]->child(rho_[0].succ());
      Edge* edgC = edg_[1]->child(rho_[1]);
      Edge* edgD = edg_[1]->child(rho_[1].succ());
      Edge* edgE = edg_[2]->child(rho_[2]);
      Edge* edgF = edg_[2]->child(rho_[2].succ());

      Vertex* vtx3 = edgD->vertex(rho_[1]);
      Vertex* vtx4 = edgF->vertex(rho_[2]);
      Vertex* vtx5 = edgB->vertex(rho_[0]);

      Edge* edgG = new Edge(*vtx3, *vtx4, attrib_);
      Edge* edgH = new Edge(*vtx4, *vtx5, attrib_);
      Edge* edgI = new Edge(*vtx5, *vtx3, attrib_);

      Triangle** tri = &chld_;

      *tri = new Triangle(*edgA, *edgH, *edgF, attrib_);
      DEBUGL(TriangleChild_D, key_ << " -- tri0: " << **tri);
      tri = &(*tri)->lnk_;
      *tri = new Triangle(*edgB, *edgC, *edgI, attrib_);
      DEBUGL(TriangleChild_D, key_ << " -- tri1: " << **tri);
      tri = &(*tri)->lnk_;
      *tri = new Triangle(*edgG, *edgD, *edgE, attrib_);
      DEBUGL(TriangleChild_D, key_ << " -- tri2: " << **tri);
      tri = &(*tri)->lnk_;
      *tri = new Triangle(*edgG, *edgH, *edgI, attrib_);
      DEBUGL(TriangleChild_D, key_ << " -- tri3: " << **tri);
    }

    Triangle* tri = chld_;
    while (i--) tri = tri->lnk_;

    return tri;
  }

  const Triangle* Triangle::child(uint i) const {
    if (i > 3) return 0;
    if (chld_) {
      Triangle* t = chld_;
      while (i--) t = t->lnk_;
      return t;
    }
    return 0;
  }
 
  std::ostream& Triangle::info(std::ostream& os) const {

    os << "Triangle(" << key_ << ", (";
    os << *edg_[0]->vertex(rho_[0]) << ", ";
    os << *edg_[1]->vertex(rho_[1]) << ", ";
    os << *edg_[2]->vertex(rho_[2]) << "), " << attrib_
//        << ", " << ((chld_ == 0) ? "no" : "has") << " children"
       << ')';
    return os;
  }

  Triangle::Triangle(Edge& edg0, Edge& edg1, Edge& edg2,
		     const Attribute attrib, const Key& key, const Z2 rho[])
    : Connector2(attrib, key), lnk_(0), chld_(0) {

    edg_[0] = &edg0; edg_[1] = &edg1; edg_[2] = &edg2;
    rho_[0] = rho[0];  rho_[1] = rho[1];  rho_[2] = rho[2];
  }

  // ****************************************************************** Quad **

  Quad::Quad(Edge& edg0, Edge& edg1, Edge& edg2, 
	     Edge& edg3, const Attribute attrib) : 
    Connector2(attrib), chld_(0), regular_(true), subdivStrategy_(0) {
    DEBUGL(QuadConstr_D, edg0 << ", " << edg1 << ", " << edg2 << ", " << edg3);
    edg_[0] = &edg0; edg_[1] = &edg1; edg_[2] = &edg2; edg_[3] = &edg3;

    int i[2];
    match(edg0, edg1, i); 

    rho_[0] = i[0] + 1;
    rho_[1] = i[1];
    rho_[2] = (edg1.vertex(rho(1).succ()) == edg2.vertex(0)) ? 0 : 1;
    rho_[3] = (edg0.vertex(rho_[0]) == edg3.vertex(0)) ? 1 : 0;
    DEBUGL(QuadConstr_D, "rho = (" << rho_[0] << ", " << rho_[1] << ", "
	   << rho_[2] << ", " << rho_[3] << ')');
    edgeAssertion_(); // Assert, that the vertices of the edges match properly
    DEBUGL(QuadConstr_D, key() << " - done.");
  }

  // Creating of intermediate cells through coalescing of two of the four childs
  Quad::Quad(const Quad& quad, uint i) : 
    Connector2(quad.attrib()), regular_(false)
  {
    DEBUGL(QuadConstr_D, key() << " - i = " << i);
    conceptsAssert(i < 4, Assertion());
    conceptsAssert(quad.getStrategy() == concepts::QuadSubdiv4::instance(),
		   Assertion());

    const concepts::Quad *children[2];

    switch(i%2){
    case 0: subdivStrategy_ = concepts::QuadSubdiv2V::instance();break;
    case 1: subdivStrategy_ = concepts::QuadSubdiv2H::instance();break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
    for(uint j=0; j < 2; ++j) {
      children[j] = quad.child((i+j)%4);

      conceptsAssert( children[j] != 0, concepts::Assertion() );
      chld_ = 
	new concepts::Joiner<Quad*, 1>(const_cast<Quad*>(children[j]), chld_);
    }

    // take edge of big quad
    // order of intermediate cells is proper choosen counter clockwise
    edg_[i] = quad.edge(i);
    rho_[i] = quad.rho(i);

    // take small edges from the children of the big quad
    // and point to the children
    for(uint j=0; j < 2; ++j) {
      children[j] = quad.child((i+j)%4);
      conceptsAssert( children[j] != 0, concepts::Assertion() );


      uint k = (i+3-2*j)%4; // child 0: (i+3)%4, child 1: (i+1)%4
      edg_[k] = children[j]->edge(k);
      rho_[k] = children[j]->rho(k);
    }

    // order of children, that first with x or y = [0..0.5]
    // neccessary for the left and right S matrices 
    if (i<2)
      chld_ = 
	new concepts::Joiner<Quad*, 1>(const_cast<Quad*>(children[0]), chld_);
    chld_ = 
      new concepts::Joiner<Quad*, 1>(const_cast<Quad*>(children[1]), chld_);
    if (i>1)
      chld_ = 
	new concepts::Joiner<Quad*, 1>(const_cast<Quad*>(children[0]), chld_);

    // create new edge in the middle
    // the edge plays no role, just for completness of the quad
    // so its not important, that there are 2 times the same edge
    // no attribute neccessary for new edge (in the interior of another edge)
    // I don't see, that setting children of the edge is needed
//     edg_[(i+2)%4] = new Edge(*children[1]->vertex((i+2)%4), 
// 			     *children[0]->vertex((i+3)%4));
    edg_[(i+2)%4] = new Edge(*children[1]->edge((i+2)%4),*children[0]->edge((i+2)%4));
    rho_[(i+2)%4] = 0;

    edgeAssertion_(); // Assert, that the vertices of the edges match properly

    DEBUGL(QuadConstr_D,
	    *edg_[0] << ", " << *edg_[1] << ", " << *edg_[2] << ", " << *edg_[3]);
    DEBUGL(QuadConstr_D, key() << " with children " << *child(0) << ", " << *child(1));
    DEBUGL(QuadConstr_D, key() << " - done.");
  }
 
  Quad::~Quad() {
    DEBUGL(QuadDestr_D, key() << " - start.");
    if (subdivStrategy_ && regular_)
      subdivStrategy_->removeChildren(*this);
    DEBUGL(QuadDestr_D, key() << " - done.");
  }

  Quad* Quad::child(uint i, bool mode) {

    DEBUGL(QuadChild_D, key_ << " - i = " << i);

    if (! mode) {
      setStrategy();
      try {
	subdivStrategy_->createChildren(*this);
      }
      catch (StrategyChange& e) {
	DEBUGL(1, e);
 	conceptsAssert(chld_ == 0, Assertion());
	subdivStrategy_ = 0; // remove strategy which was set before
	throw e;
      }
    }

#if QuadChild_D
    {
      std::auto_ptr<Scan<Quad> > sc(children_());
      while (*sc) {
	DEBUGL(1, "child = " << (*sc)++);
      }
    }
#endif

    uint number;
    if (subdivStrategy_ == concepts::QuadSubdiv4::instance()) {
      number = 3;
    }
    else number = 1;
    if (i > number) return 0;      

    // children are sorted vice versa
    // right order important e.g. for subdivision of hexahedra
    uint j = number - i;

    std::auto_ptr<Scan<Quad> > sc(children_());
    DEBUGL(QuadChild_D, " i = " << i + j);
    Quad* qd = &(*sc)++;
    while (j--) {
       DEBUGL(QuadChild_D, " i = " << i + j);
       if (*sc) qd = &(*sc)++;
       else {
	 DEBUGL(QuadChild_D, " not found");
	 return 0;
       }
    }

    return qd;
  }

  const Quad* Quad::child(uint i) const {

    DEBUGL(QuadChild_D, key_ << " - i = " << i);

    // children are sorted vice versa
    // right order important e.g. for subdivision of hexahedra
    if (subdivStrategy_ == concepts::QuadSubdiv4::instance()) {
      if (i > 3) return 0;      
      i = 3 - i;
    } else {
      if (i > 1) return 0;
      i = 1 - i;
    }

    std::auto_ptr<Scan<Quad> > sc(children_());
    Quad* qd = &(*sc)++;
    while (i--)
      if (*sc) qd = &(*sc)++;
      else return 0;

    return qd;
  }

  void Quad::setStrategy(const QuadSubdivision* strategy)
    throw(StrategyChange) {
    if ( (subdivStrategy_) && (strategy == 0) )
      return;
    if ( (strategy) && (subdivStrategy_) && (chld_) &&
	 (strategy != subdivStrategy_) )
      throw conceptsException(StrategyChange(*subdivStrategy_, *strategy,
					     *this));
    if (! strategy)
      strategy = QuadSubdiv4::instance();
    DEBUGL(QuadsetStrategy_D, "setting strategy to " << *strategy << " in "
	   << *this);
    subdivStrategy_ = strategy;
  }

  std::ostream& Quad::info(std::ostream& os) const {

    os << "Quad(" << key_ << ", (";
    os << *edg_[0]->vertex(rho_[0]) << ", ";
    os << *edg_[1]->vertex(rho_[1]) << ", ";
    os << *edg_[2]->vertex(rho_[2]) << ", ";
    os << *edg_[3]->vertex(rho_[3])
       << ", " << attrib_
//        << ", " << ((chld_ == 0) ? "no" : "has") << " children"
       << ')';
    return os;
  }

// Assert, that the vertices of the four edges match properly
  void Quad::edgeAssertion_() const {
    for(uint i=0, j=1; i < 4; ++i,j=(i+1)%4)
      conceptsAssert3(edg_[i]->vertex(rho_[i].succ())== edg_[j]->vertex(rho_[j]),
		      Assertion(), "edges: " << *edg_[0] << ", " << *edg_[1] <<
		      ", " << *edg_[2] << ", " << *edg_[3]);
  }

  Quad::Quad(Edge& edg0, Edge& edg1, Edge& edg2, Edge& edg3,
	     const Attribute attrib, const Key& key, const Z2 rho[]) :
    Connector2(attrib, key), chld_(0), subdivStrategy_(0) {

    edg_[0] = &edg0; edg_[1] = &edg1; edg_[2] = &edg2; edg_[3] = &edg3;
    rho_[0] = rho[0];  rho_[1] = rho[1];  rho_[2] = rho[2];  rho_[3] = rho[3];
    DEBUGL(QuadConstr_D, *edg_[0] << ", rho = " << rho_[0]);
    DEBUGL(QuadConstr_D, *edg_[1] << ", rho = " << rho_[1]);
    DEBUGL(QuadConstr_D, *edg_[2] << ", rho = " << rho_[2]);
    DEBUGL(QuadConstr_D, *edg_[3] << ", rho = " << rho_[3]);
    conceptsAssert(*edg_[0]->vertex(rho_[0].succ()) ==
		   *edg_[1]->vertex(rho_[1]), Assertion());
    conceptsAssert(*edg_[1]->vertex(rho_[1].succ()) ==
		   *edg_[2]->vertex(rho_[2]), Assertion());
    conceptsAssert(*edg_[2]->vertex(rho_[2].succ()) ==
		   *edg_[3]->vertex(rho_[3]), Assertion());
    conceptsAssert(*edg_[3]->vertex(rho_[3].succ()) ==
		   *edg_[0]->vertex(rho_[0]), Assertion());
  }

} // namespace concepts
