/* Topology.
 * how to subdivide, now in 3D
 */

#include <sstream>
#include <algorithm>
#include "basics/debug.hh"
#include "basics/exceptions.hh"
#include "topology3D.hh"

// debugging match
#define matchT_D 0

// debugging Tetrahedron
#define geoTetrahedronConstr_D 0
#define geoTetrahedronDestr_D 0
#define geoTetrahedronVertex_D 0
#define geoTetrahedronEdge_D 0
#define geoTetrahedronChildGen_D 0
#define geoTetrahedronChildSmall_D 0
#define geoTetrahedronChildTed_D 0

// debugging Hexahedron
#define HexahedronConstr_D 0
#define HexahedronDestr_D 0
#define HexahedronChild_D 0
#define HexahedronsetStrategy_D 0

namespace concepts {

  // ***************************************************************** match **

  int match(const Triangle& tri0, const Triangle& tri1,
	    int idx[]) {

    DEBUGL(matchT_D, "tri0 = " << tri0);
    DEBUGL(matchT_D, "tri1 = " << tri1);
    for (uint i = 0; i < 3; ++i)
      for (uint j = 0; j < 3; ++j)
	if (tri0.edge(i) == tri1.edge(j)) {
	  idx[0] = i;
	  idx[1] = j;
	  DEBUGL(matchT_D, "idx[0] = " << idx[0] << ", idx[1] = " << idx[1]);
	  return 1;
	}
    return 0;
  }

  int match(const Quad& quad0, const Quad& quad1, int idx[]) {

    for (uint i = 0; i < 4; ++i)
      for (uint j = 0; j < 4; ++j)
	if (quad0.edge(i) == quad1.edge(j)) {
	  idx[0] = i;
	  idx[1] = j;
	  return 1;
	}
    return 0;
  }

  // *********************************************************** Tetrahedron **

  Tetrahedron::Tetrahedron(Triangle& tri0, Triangle& tri1, 
			   Triangle& tri2, Triangle& tri3, Attribute attrib) : 
    Connector3(attrib) {

    DPL(geoTetrahedronConstr_D, '[' <<  __FILE__ << ", line " 
	<< __LINE__ << "] Tetrahedron::Tetrahedron -- key =", key_);

    tri_[0] = &tri0; tri_[1] = &tri1; tri_[2] = &tri2; tri_[3] = &tri3;
    chld_ = lnk_ = NULL;

    int i[4];
    match(tri0, tri1, i);
    match(tri2, tri3, i + 2);

    for (uint j = 0; j < 4; ++j)
      tau_[j] = i[j];

    Vertex* vtx0 = tri3.vertex(tau_[3].pred());
    DPL(geoTetrahedronConstr_D, '[' <<  __FILE__ << ", line " 
	<< __LINE__ << "] Tetrahedron::Tetrahedron -- vtx0 =", *vtx0);
    DPL(geoTetrahedronConstr_D, '[' <<  __FILE__ << ", line " 
	<< __LINE__ << "] Tetrahedron::Tetrahedron -- tri0 =", tri0);
    rho_[0] = tri0.vertex(tau_[0]) == vtx0 ? 0 : 1;
    Z3 tmp = tau_[1];
    rho_[1] = tri1.vertex(tmp) == vtx0 ? 0 : 1;
 
    Vertex* vtx2 = tri0.vertex(tau_[0].pred());
    DPL(geoTetrahedronConstr_D, '[' <<  __FILE__ << ", line " 
	<< __LINE__ << "] Tetrahedron::Tetrahedron -- vtx1 =", *vtx2);
    tmp = tau_[2];
    rho_[2] = tri2.vertex(tmp) == vtx2 ? 0 : 1;
    tmp = tau_[3];
    rho_[3] = tri3.vertex(tmp) == vtx2 ? 1 : 0;

    if (rho_[1]) ++tau_[1]; else --tau_[1];
    if (rho_[2]) --tau_[2]; else ++tau_[2];
  }

  Tetrahedron::~Tetrahedron() {
  
    if (chld_ != 0) {
      Tetrahedron* tetra = chld_; chld_ = tetra->lnk_;
      delete tetra->tri_[2];
      delete tetra;

      tetra = chld_; chld_ = tetra->lnk_;
      delete tetra->tri_[3];
      delete tetra;

      tetra = chld_; chld_ = tetra->lnk_;
      delete tetra->tri_[1];
      delete tetra;

      tetra = chld_; chld_ = tetra->lnk_;
      delete tetra->tri_[0];
      delete tetra;

      tetra = chld_; chld_ = tetra->lnk_;
      DPL(geoTetrahedronDestr_D, '[' <<  __FILE__ << ", line " 
	  << __LINE__ << "] Tetrahedron::~Tetrahedron -- deleting",
	  *(tetra->edge(4)));
      delete tetra->edge(4);
      delete tetra->tri_[2];
      delete tetra;

      tetra = chld_; chld_ = tetra->lnk_;
      delete tetra->tri_[1];
      delete tetra;

      tetra = chld_; chld_ = tetra->lnk_;
      delete tetra->tri_[0];
      delete tetra;

      delete chld_->tri_[3];
      delete chld_;
    }
  }

  Vertex* Tetrahedron::vertex(uint i) const {

    DPL(geoTetrahedronVertex_D, '[' <<  __FILE__ << ", line " 
	<< __LINE__ << "] Tetrahedron::vertex -- i =", i);
#if geoTetrahedronVertex_D
    for (uint j = 0; j < 4; ++j) {
      DPL(geoTetrahedronVertex_D, '[' <<  __FILE__ << ", line " 
	  << __LINE__ << "] Tetrahedron::vertex -- tau[" << j << "] = " <<
	  tau_[j] << ", rho[" << j << "] =", rho_[j]);
    }
#endif

    if (i > 3)
      return 0;

    else if (i == 3) {

      DPL(geoTetrahedronVertex_D, '[' <<  __FILE__ << ", line " 
	  << __LINE__ << "] Tetrahedron::vertex -- tri_[1] =", *tri_[1]);

      if (rho_[1])
	return tri_[1]->vertex(tau_[1].succ());
      else
	return tri_[1]->vertex(tau_[1]);
    } else {

      DPL(geoTetrahedronVertex_D, '[' <<  __FILE__ << ", line " 
	  << __LINE__ << "] Tetrahedron::vertex -- tri_[" << i << "] =", 
	  *tri_[i]);

      ++i;
      if (rho_[i])
	return tri_[i]->vertex(tau_[i]);
      else
	return tri_[i]->vertex(tau_[i].succ());
    }
  }

  Edge* Tetrahedron::edge(uint i) const {

    DPL(geoTetrahedronEdge_D, '[' <<  __FILE__ << ", line " 
	<< __LINE__ << "] Tetrahedron::edge -- i =", i);

    if (i > 5)
      return 0;

    else {
      if (i < 3) {

	++i;
	if (rho_[i])
	  return tri_[i]->edge(tau_[i].pred());
	else
	  return tri_[i]->edge(tau_[i].succ());

      } else {

	i -= 2;
	return tri_[i]->edge(tau_[i]);
      }
    }
  }

  Tetrahedron* Tetrahedron::child(uint i, bool mode) {

    DEBUGL(geoTetrahedronChildGen_D, '[' << key_ << "i = " << i);

    if (i > 7) return 0;

    if (mode) {
      if (chld_) {
	Tetrahedron* tetra = chld_;
	while (i--) tetra = tetra->lnk_;
	return tetra;
      }
      return 0;
    }

    if (chld_ == 0) {
      DEBUGL(geoTetrahedronChildGen_D, '[' << key_ << "] subdiving " << *this);
      Triangle* triA = tri_[0]->child(tau_[0]);
      Triangle* triB = tri_[0]->child(tau_[0].succ());
      Triangle* triC = tri_[0]->child(tau_[0].pred());
      Triangle* triD = tri_[0]->child(3);

      if (rho_[0]) { Triangle* tri = triB; triB = triA; triA = tri; }

      Triangle* triE = tri_[1]->child(tau_[1]);
      Triangle* triF = tri_[1]->child(tau_[1].succ());
      Triangle* triG = tri_[1]->child(tau_[1].pred());
      Triangle* triH = tri_[1]->child(3);

      if (rho_[1]) { Triangle* tri = triF; triF = triE; triE = tri; }

      Triangle* triI = tri_[2]->child(tau_[2]);
      Triangle* triJ = tri_[2]->child(tau_[2].succ());
      Triangle* triK = tri_[2]->child(tau_[2].pred());
      Triangle* triL = tri_[2]->child(3);
    
      if (rho_[2]) { Triangle* tri = triJ; triJ = triI; triI = tri; }
  
      Triangle* triM = tri_[3]->child(tau_[3]);
      Triangle* triN = tri_[3]->child(tau_[3].succ());
      Triangle* triO = tri_[3]->child(tau_[3].pred());
      Triangle* triP = tri_[3]->child(3);

      if (rho_[3]) { Triangle* tri = triN; triN = triM; triM = tri; }

      Edge* edgA = triD->edge(tau_[0].succ());
      Edge* edgB = triD->edge(tau_[0].pred());
      Edge* edgC = triD->edge(tau_[0]);

      if (rho_[0]) { Edge* edg = edgB; edgB = edgA; edgA = edg; }

      Edge* edgD = triH->edge(tau_[1].succ());       
      Edge* edgE = triH->edge(tau_[1].pred());
      Edge* edgF = triH->edge(tau_[1]);

      if (rho_[1]) { Edge* edg = edgE; edgE = edgD; edgD = edg; }

      Edge* edgG = triL->edge(tau_[2].succ());       
      Edge* edgH = triL->edge(tau_[2].pred());
      Edge* edgI = triL->edge(tau_[2]);

      if (rho_[2]) { Edge* edg = edgH; edgH = edgG; edgG = edg; }

      Edge* edgJ = triP->edge(tau_[3].succ());       
      Edge* edgK = triP->edge(tau_[3].pred());
      Edge* edgL = triP->edge(tau_[3]);

      if (rho_[3]) { Edge* edg = edgK; edgK = edgJ; edgJ = edg; }

      Edge* edgM = new Edge(*edge(2)->child(0)->vertex(1),
				    *edge(4)->child(0)->vertex(1), attrib_);
      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] edgM: " << *edgM);

      Triangle* triQ = new Triangle(*edgA, *edgE, *edgL, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triQ: " << *triQ);
      Triangle* triR = new Triangle(*edgB, *edgH, *edgF, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triR: " << *triR);
      Triangle* triS = new Triangle(*edgC, *edgK, *edgI, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triS: " << *triS);
      Triangle* triT = new Triangle(*edgD, *edgG, *edgJ, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triT: " << *triT);
      Triangle* triU = new Triangle(*edgM, *edgD, *edgL, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triU: " << *triU);
      Triangle* triV = new Triangle(*edgM, *edgC, *edgH, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triV: " << *triV);
      Triangle* triW = new Triangle(*edgM, *edgG, *edgK, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triW: " << *triW);
      Triangle* triX = new Triangle(*edgM, *edgA, *edgF, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] triX: " << *triX);

      Tetrahedron** tetra = &chld_;

      *tetra = new Tetrahedron(*triA, *triF, *triQ, *triO, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted0: " << **tetra);
      tetra = &(*tetra)->lnk_;
      *tetra = new Tetrahedron(*triB, *triG, *triJ, *triR, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted1: " << **tetra);
      tetra = &(*tetra)->lnk_;
      *tetra = new Tetrahedron(*triC, *triS, *triK, *triN, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted2: " << **tetra);
      tetra = &(*tetra)->lnk_;
      *tetra = new Tetrahedron(*triT, *triE, *triI, *triM, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted3: " << **tetra);
      tetra = &(*tetra)->lnk_;
      *tetra = new Tetrahedron(*triQ, *triX, *triU, *triH, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted4: " << **tetra);
      tetra = &(*tetra)->lnk_;

      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] child: 5");

      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri0: " << *triD);
      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri1: " << *triX);
      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri2: " << *triR);
      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri3: " << *triV);

      *tetra = new Tetrahedron(*triD, *triX, *triR, *triV, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted5: " << **tetra);
      tetra = &(*tetra)->lnk_;

      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] child: 6");

      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri0: " << *triV);
      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri1: " << *triS);
      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri2: " << *triW);
      DEBUGL(geoTetrahedronChildSmall_D, '[' << key_ << "] tri3: " << *triL);

      *tetra = new Tetrahedron(*triV, *triS, *triW, *triL, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted6: " << **tetra);
      tetra = &(*tetra)->lnk_;
      *tetra = new Tetrahedron(*triU, *triP, *triT, *triW, attrib_);
      DEBUGL(geoTetrahedronChildTed_D, '[' << key_ << "] ted7: " << **tetra);
    }

    Tetrahedron* tetra = chld_;
    while (i--) tetra = tetra->lnk_;

    return tetra;
  }

  const Tetrahedron* Tetrahedron::child(uint i) const {
    if (i > 7) return 0;
    if (chld_) {
      Tetrahedron* tetra = chld_;
      while (i--) tetra = tetra->lnk_;
      return tetra;
    }
    return 0;
  }

  std::ostream& Tetrahedron::info(std::ostream& os) const {

    os << "Tetrahedron(" << key_ << ", (";
  
    for (uint i = 0; i < 4; ++i) {
      os << *(vertex(i));
      if (i < 3)
	os << ", ";
      else
	os << ")";
    }
    return os << ", " << attrib_
// 	      << ", " << ((chld_ == 0) ? "no" : "has") << " children"
	      << ')';
  }

  // ************************************************************ Hexahedron **

  Hexahedron::Hexahedron(Quad& quad0, Quad& quad1, Quad& quad2, Quad& quad3, 
			 Quad& quad4, Quad& quad5, const Attribute attrib) :
    Connector3(attrib), chld_(0), lnk_(0), subdivStrategy_(0) {

    DEBUGL(HexahedronConstr_D, "quads = " << quad0 << ", " << quad1 << ", "
	   << quad2 << ", " << quad3 << ", " << quad4 << ", " << quad5);

    quad_[0] = &quad0; quad_[1] = &quad1; quad_[2] = &quad2; quad_[3] = &quad3;
    quad_[4] = &quad4; quad_[5] = &quad5;

    int i[6];
    match(quad0, quad1, i);
    match(quad2, quad3, i + 2);
    match(quad4, quad5, i + 4);

    // tau 0, 3
    for (uint j = 0; j < 6; ++j)
      tau_[j] = i[j];
#if HexahedronConstr_D
    for (uint j = 0; j < 6; ++j)
      DEBUGL(HexahedronConstr_D, "match[" << j << "] = " << tau_[j]);
#endif

    // rho and tau 2
    int k = -1;
    ++tau_[2];
    ++tau_[2];
    Vertex* vtx0 = quad2.vertex(tau_[2]);
    DEBUGL(HexahedronConstr_D, "vtx0 = " << *vtx0); 
    rho_[2] = 0;
    for (uint j = 0; j < 4; ++j)
      if (vtx0 == quad0.vertex(j)) {
	rho_[2] = 1;
	k = j;
      }
    DEBUGL(HexahedronConstr_D, "rho[2] = " << rho_[2] << ", k = " << k);
    if (rho_[2] == 0) {
      vtx0 = quad2.vertex(tau_[2].succ());
      for (uint j = 0; j < 4; ++j)
	if (vtx0 == quad0.vertex(j))
	  k = j;
    }
    DEBUGL(HexahedronConstr_D, "rho[2] = " << rho_[2] << ", k = " << k);
    DEBUGL(HexahedronConstr_D, "vtx0 = " << *vtx0); 
    conceptsAssert(k >= 0, Assertion());

    // rho 0
    if (k == tau_[0])
      rho_[0] = 1;
    else
      rho_[0] = 0;

    // rho 3
    Vertex* vtx3;
    if (rho_[0] == 0)
      vtx3 = quad0.vertex(((Z4)tau_[0].succ()).succ());
    else
      vtx3 = quad0.vertex(tau_[0].pred());
    if (vtx3 == quad3.vertex(tau_[3]))
      rho_[3] = 1;
    else 
      rho_[3] = 0;

    // rho, tau 1
    Vertex* vtx1;
    if (rho_[0] == 0)
      vtx1 = quad0.vertex(tau_[0]);
    else
      vtx1 = quad0.vertex(tau_[0].succ());
    if (vtx1 == quad1.vertex(tau_[1])) {
      rho_[1] = 0;
      --tau_[1];
    } else {
      rho_[1] = 1;
      ++tau_[1];
    }

    // rho, tau 4
    if (vtx1 == quad4.vertex(tau_[4].pred())) {
      rho_[4] = 0;
      ++tau_[4];
    } else {
      rho_[4] = 1;
      --tau_[4];
    }

    // rho, tau 5
    Vertex* vtx5;
    if (rho_[1] == 0)
      vtx5 = quad1.vertex(tau_[1]);
    else
      vtx5 = quad1.vertex(tau_[1].succ());
    DEBUGL(HexahedronConstr_D, "vtx5 = " << *vtx5); 
    if (vtx5 == quad5.vertex(tau_[5])) {
      rho_[5] = 0;
      --tau_[5];
    } else {
      rho_[5] = 1;
      ++tau_[5];
    }

#if HexahedronConstr_D
    for (uint j = 0; j < 6; ++j)
      DEBUGL(HexahedronConstr_D, "tau[" << j << "] = " << tau_[j]
	     << ", rho[" << j << "] = " << rho_[j]);
#endif

#if __GNUC__ // Sun Workshop makes troubles
    conceptsAssert(quad_[0]->edge(tau_[0]) ==
		   quad_[1]->edge( (tau_[1]+(rho_[1]?3:1))%4 ), Assertion());
    conceptsAssert(quad_[0]->edge( (tau_[0]+(rho_[0]?3:1))%4 ) ==
		   quad_[2]->edge( (tau_[2]+(rho_[2]?3:1))%4 ), Assertion());
    conceptsAssert(quad_[0]->edge( (tau_[0]+2)%4 ) ==
		   quad_[3]->edge( (tau_[3]+(rho_[3]?3:1))%4 ), Assertion());
    conceptsAssert(quad_[0]->edge( (tau_[0]+(rho_[0]?1:3))%4 ) ==
		   quad_[4]->edge( (tau_[4]+(rho_[4]?3:1))%4 ), Assertion());

    conceptsAssert(quad_[5]->edge(tau_[5]) ==
		   quad_[1]->edge( (tau_[1]+(rho_[1]?1:3))%4 ), Assertion());
    conceptsAssert(quad_[5]->edge( (tau_[5]+(rho_[5]?1:3))%4 ) ==
		   quad_[2]->edge( (tau_[2]+(rho_[2]?1:3))%4 ), Assertion());
    conceptsAssert(quad_[5]->edge( (tau_[5]+2)%4 ) ==
		   quad_[3]->edge( (tau_[3]+(rho_[3]?1:3))%4 ), Assertion());
    conceptsAssert(quad_[5]->edge( (tau_[5]+(rho_[5]?3:1))%4 ) ==
		   quad_[4]->edge( (tau_[4]+(rho_[4]?1:3))%4 ), Assertion());

    conceptsAssert(quad_[1]->edge( (tau_[1]+2)%4 ) ==
		   quad_[2]->edge(tau_[2]), Assertion());
    conceptsAssert(quad_[2]->edge( (tau_[2]+2)%4 ) ==
		   quad_[3]->edge(tau_[3]), Assertion());
    conceptsAssert(quad_[3]->edge( (tau_[3]+2)%4 ) ==
		   quad_[4]->edge(tau_[4]), Assertion());
    conceptsAssert(quad_[4]->edge( (tau_[4]+2)%4 ) ==
		   quad_[1]->edge(tau_[1]), Assertion());
#endif
  }

  Hexahedron::~Hexahedron() {
    if (subdivStrategy_)
      subdivStrategy_->removeChildren(*this);
  }

  Vertex* Hexahedron::vertex(uint i) const {

    if (i > 7)
      return 0;

    if (i < 4) {
      ++i;
      if (rho_[i])
	return quad_[i]->vertex(tau_[i]);
      else
	return quad_[i]->vertex(tau_[i].succ());
    } else {
      i -= 3;
      if (rho_[i])
	return quad_[i]->vertex(tau_[i].succ());
      else
	return quad_[i]->vertex(tau_[i]);
    }
  }

  Edge* Hexahedron::edge(uint i) const {

    if (i > 11)
      return 0;

    if (i < 4) {
      ++i;
      if (rho_[i])
	return quad_[i]->edge(tau_[i].pred());
      else
	return quad_[i]->edge(tau_[i].succ());
    } else {
      if (i < 8) {
	i -= 3;
	return quad_[i]->edge(tau_[i]);
      } else {
	i -= 7;
	if (rho_[i])
	  return quad_[i]->edge(tau_[i].succ());
	else
	  return quad_[i]->edge(tau_[i].pred());
      }
    }
  }

  Hexahedron* Hexahedron::child(uint i, bool mode) {

    DEBUGL(HexahedronChild_D, "i = " << i);

    if (i > 7) return 0;

    if (! mode) {
      setStrategy();
      try {
	subdivStrategy_->createChildren(*this);
      }
      catch (StrategyChange& e) {
// 	DEBUGL(1, e);
	conceptsAssert(chld_ == 0, Assertion());
	subdivStrategy_ = 0; // remove strategy which was set before
	throw e;
      }
    }

    Hexahedron *hex = chld_;
    while (i--)
      if (hex) hex = hex->lnk_;
      else return 0;

    return hex;
  }

  const Hexahedron* Hexahedron::child(uint i) const {
    if (i > 7) return 0;
    Hexahedron *hex = chld_;
    while (i--)
      if (hex) hex = hex->lnk_;
      else return 0;
    return hex;
  }

  void Hexahedron::setStrategy(const HexSubdivision* strategy)
    throw(StrategyChange) {
    if ( (subdivStrategy_) && (strategy == 0) )
      return;
    if ( (strategy) && (subdivStrategy_) && (chld_) &&
	 (strategy != subdivStrategy_) )
      throw conceptsException(StrategyChange(*subdivStrategy_, *strategy,
					     *this));
    if (! strategy)
      strategy = HexSubdiv8::instance();
    DEBUGL(HexahedronsetStrategy_D, "setting strategy to " << *strategy
	   << " in " << *this);
    subdivStrategy_ = strategy;
  }

  std::ostream& Hexahedron::info(std::ostream& os) const {

    os << "Hexahedron(" << key_ << ", (";

    for (uint i = 0; i < 8; ++i) {
      os << *(vertex(i)) << ", ";
    }
    os << attrib_
//        << ", " << ((chld_ == 0) ? "no" : "has") << " children"
       << ')';
    return os;
  }

} // namespace concepts
