#ifndef uniformlyRefinedMesh2_hh
#define uniformlyRefinedMesh2_hh

#include "geometry/mesh.hh"
#include <vector>

namespace concepts {

  /** Wrapper class refining an existing 2d mesh uniformly. This class then
      represents a mesh where each cell of the original mesh is subdivided
      into 4<sup>level</sup> cells.
      @author Christoph Winkelmann, 2003
      @ingroup geometry
  */
  class UniformlyRefinedMesh2 : public Mesh2 {
  public:
    /** Constructor. Takes an existing mesh to be refined by a given \c level
        @param coarseMesh Mesh to be refined
        @param level Desired level of refinement
    */
    UniformlyRefinedMesh2(Mesh2& coarseMesh, uint level);
    /// Destructor
    virtual ~UniformlyRefinedMesh2() {}
    /// Returns the number of cells in the mesh.
    virtual uint ncell() const { return cell_.size(); }
    /// Returns a scanner over the cells of the mesh.
    virtual Scan2* scan() { return new S(cell_); }
  private:
    /// Scanner class for UniformlyRefindeMesh2
    class S : public Scan<Cell2> {
    public:
      inline S(std::vector<Cell2*>& cell) :
        idx_(cell.begin()), cell_(cell) {}
      inline S(const S& scan) : idx_(scan.idx_), cell_(scan.cell_) {}
      inline bool eos() const { return idx_ == cell_.end(); }
      inline Cell2& operator++(int) { return *(*idx_++); }
      inline Scan2* clone() const { return new S(*this); }
    private:
      std::vector<Cell2*>::iterator idx_;
      std::vector<Cell2*>& cell_;
    };
    void addCell(Cell2* cell, uint level);
    std::vector<Cell2*> cell_;
  };

} // namespace concepts

#endif //uniformlyRefinedMesh2_hh
