/* graphics of the mesh in Gnuplot
 */

#ifndef graphMeshGnuplot_hh
#define graphMeshGnuplot_hh

#include <string>
#include <fstream>
#include <memory>
#include "basics/typedefs.hh"
#include "basics/vectorsMatricesForward.hh"
#include "basics/exceptions.hh"
#include "space/postProcess.hh"

namespace graphics {
  using concepts::Real;

  // ******************************************************* MeshGnuplotCell **

  /** Writes the data which describe the elements to the given streams.
      @author Philipp Frauenfelder, 2002.
  */
  template<typename F>
  class MeshGnuplotCell : public concepts::CellPostprocess<F> {
  public:
    MeshGnuplotCell(std::ofstream* ofs) : ofs_(ofs) {}
    virtual ~MeshGnuplotCell();
    virtual void operator() (const concepts::Element<F>& elm)
      throw(concepts::MissingFeature);
    virtual void operator() (const concepts::Cell& cell)
      throw(concepts::MissingFeature);
    std::ofstream* write() { return ofs_; }
  private:
    std::ofstream* ofs_;
  };

  // *********************************************************** MeshGnuplot **

  /** Creates a data file for viewing the mesh with Gnuplot using
      MeshGnuplotCell.
      @author Philipp Frauenfelder, 2002
  */
  template<typename T>
  void drawMeshGnuplot(T& msh_spc, std::string filename) {
    std::ofstream ofs(filename.c_str());
    concepts::GlobalPostprocess<Real> postProcess(msh_spc);
    MeshGnuplotCell<Real> gnu(&ofs);
    postProcess(gnu);
  }

} // namespace graphics

#endif // graphMeshGnuplot_hh
