/* takes a topology (mesh or space) and generates a file for dot/dotty
 */

#include "topologyDotty.hh"
#include "space/element.hh"
#include "geometry/cell.hh"
#include "toolbox/elementGraphics.hh"

namespace graphics {

  // ********************************************************* TopologyDotty **

  template<class F>
  TopologyDotty<F>::TopologyDotty(concepts::Mesh& msh, std::string filename) :
    spc_(0), msh_(&msh), filename_(filename) {

    concepts::GlobalPostprocess<F> post(*msh_);
    TopologyDottyCell<F> td(filename_);
    post(td);
  }

  template<class F>
  TopologyDotty<F>::TopologyDotty(concepts::Space<F>& spc,
				  std::string filename) :
    spc_(&spc), msh_(0), filename_(filename) {

    concepts::GlobalPostprocess<F> post(*spc_);
    TopologyDottyCell<F> td(filename_);
    post(td);
  }

  template class TopologyDotty<Real>;

  // ***************************************************** TopologyDottyCell **

  template<class F>
  TopologyDottyCell<F>::TopologyDottyCell(const std::string file) :
    os_(file.c_str()) {
    os_ << "graph G {" << std::endl
	<< "  concentrate = true" << std::endl;
  }

  template<class F>
  TopologyDottyCell<F>::~TopologyDottyCell() {
    os_ << "}" << std::endl;
  }

  template<class F>
  void TopologyDottyCell<F>::operator() (const concepts::Element<F>& elm) {
    elm.graphics()->operator()(elm,
			       concepts::ElementGraphics<Real>::TOPOLOGY_DOTTY,
			       *this);
  }

  template<class F>
  void TopologyDottyCell<F>::operator() (const concepts::Cell& cell) {
    std::cout << cell << std::endl;

    const concepts::Cell1* c1 = dynamic_cast<const concepts::Cell1*>(&cell);
    if (c1) {
      edge_(c1->connector());
      return;
    }

    const concepts::Cell2* c2 = dynamic_cast<const concepts::Cell2*>(&cell);
    if (c2) {
      const concepts::Connector2& cntr = c2->connector();
      const concepts::Connector1* e = 0;
      for (uint i = 0; (e = cntr.edge(i)) != 0; ++i)
	edge_(*e);
      return;
    }

    const concepts::Cell3* c3 = dynamic_cast<const concepts::Cell3*>(&cell);
    if (c3) {
      const concepts::Connector3& cntr = c3->connector();
      const concepts::Connector1* e = 0;
      for (uint i = 0; (e = cntr.edge(i)) != 0; ++i)
	edge_(*e);
      return;
    }

    throw conceptsException(concepts::MissingFeature("cell not supported"));
  }

  template<class F>
  void TopologyDottyCell<F>::edge_(const concepts::Connector1& cntr) {
      os_ << "    " << (uint)(cntr.vertex(0)->key())
	  << " -- " << (uint)(cntr.vertex(1)->key()) << ';' << std::endl;
  }

  template class TopologyDottyCell<Real>;

} // namespace graphics
