/* takes a topology (mesh or space) and generates a file for dot/dotty
 */

#ifndef topologyDotty_hh
#define topologyDotty_hh

#include <string>
#include <fstream>
#include "space/postProcess.hh"
#include "basics/typedefs.hh"

namespace concepts {
  // forward declarations
  template<class F>
  class Element;

  template<class F>
  class Space;

  class Cell;
  class Connector1;
  class Mesh;
}

namespace graphics {

  using concepts::Real;

  // ********************************************************* TopologyDotty **

  /** Executor class for TopologyDottyCell.

      A graph where the vertices are the vertices of the mesh and the edges
      are the edges of the mesh is written to the file.
      The resulting file can be used as input for dot, dotty or
      neato. neato gives very good results.

      @see <a href="http://www.research.att.com/sw/tools/graphviz/">homepage of
      Graphviz at AT&T Labs-Research</a>
      @author Philipp Frauenfelder, 2002
   */
  template<class F>
  class TopologyDotty {
  public:
    /** Constructor. The coarsest grid will be depicted. To store the results,
	a new file is created.
    */
    TopologyDotty(concepts::Mesh& msh, std::string filename);

    /** Constructor. The currently used grid in the space will be depicted.
	To store the results, a new file is created.
    */
    TopologyDotty(concepts::Space<F>& spc, std::string filename);
  private:
    /// The space
    concepts::Space<F>* spc_;

    /// The mesh
    concepts::Mesh* msh_;

    /// Filename
    std::string filename_;
  };

  // ***************************************************** TopologyDottyCell **

  /** Produces an input file for dot / dotty to display the topology as
      a directed graph.

      @see <a href="http://www.research.att.com/sw/tools/graphviz/">homepage of
      Graphviz at AT&T Labs-Research</a>
      @author Philipp Frauenfelder, 2001
  */
  template<class F>
  class TopologyDottyCell : public concepts::CellPostprocess<F> {
  public:
    /** Constructor.
	@param file Name of the file where the output should be stored
    */
    TopologyDottyCell(const std::string file);
    ~TopologyDottyCell();
    virtual void operator() (const concepts::Element<F>& elm);
    virtual void operator() (const concepts::Cell& cell);
  private:
    void edge_(const concepts::Connector1& cntr);

    /// Stream to the file
    std::ofstream os_;
  };

} // namespace graphics

#endif // topologyDotty_hh
