/* class for the hp FEM space in 1D
 */

#include "space.hh"
#include "toolbox/scannerConnectors.hh"
#include "basics/debug.hh"

// debugging

#define buildElms1D_D 0

namespace hp1D {

  // ***************************************************************** Space **

  void Space::buildElements_(concepts::Edge1d& cell,
			     concepts::TColumn<Real>* T0) {
    concepts::Edge& cntr = cell.connector();
    concepts::AdaptiveControlP<1>& ctrl1(ctrl1_[cntr.key()]);
    DEBUGL(buildElms1D_D, cell << " has " << ctrl1);

    // maximal number of degrees of freedom in this element
    uint nn = ctrl1.p_[0]+1; // p max
    concepts::TColumn<Real>* T1 = 0;

    if (ctrl1.tag_.isMember()) {
      // k counts the local degrees of freedom in topological notation
      uint k = 0;
      // loop over vertices
      concepts::Vertex* cntr0 = 0;
      for(uint i = 0; (cntr0 = cntr.vertex(i)) != 0; ++i, ++k) {
	concepts::AdaptiveControl& ctrl0(ctrl0_[cntr0->key()]);
	if (ctrl0.tag_.isMember() || ctrl0.tag_.isPassive()) {
	  // is member of the space or active
	  if (ctrl0.tag_.isMember() && ctrl0.tag_.isActive()) {
	    // active member of the space
	    if (! ctrl0.tag_.idxValid()) { 
	      // not yet counted
	      ctrl0.idx_ = dim_; ++dim_;
	      ctrl0.tag_.validateIdx();
	    }
	    T1 = new concepts::TColumn<Real>(nn, ctrl0.idx_, T1);

	    DEBUGL(buildElms1D_D, "new T col " << *T1 << ", k = " << k
		   << ", nn = " << nn << ", for " << *cntr0);
	    (*T1)[k] = 1.0;
	    DEBUGL(buildElms1D_D, "T col " << *T1 << " from " << *cntr0);
	  } // if member of the space and active
	} // if member of the space or passive
      } // for vertices

      if (ctrl1.tag_.isMember()) {
	if (ctrl1.p_[0] > 1) { // p int
	  ctrl1.idx_ = dim_;
	  dim_ += (ctrl1.p_[0]-1); // p int
	  ctrl1.tag_.validateIdx();

	  uint idx = ctrl1.idx_;
	  for (uint i = 2; i < (uint)ctrl1.p_[0]+1; ++i) { // p int
	    T1 = new concepts::TColumn<Real>(nn, idx++, T1);
	    (*T1)[i] = 1.0;
	  } // for i
	}
      }

      elm_ = new concepts::Joiner<Element<Real>*, 1>
	(new Element<Real>(cell, ctrl1.p_[0], T0, T1), elm_);
      ++nelm_;
    } else {
      concepts::Edge1d* chld = 0;
      for(uint i = 0; (chld = cell.child(i)) != 0; ++i) {
	buildElements_(*chld, T0);
      } // for i
    }

    while (T1 != 0) {
      concepts::TColumn<Real>* foo = T1; T1 = T1->link(); delete foo;
    }
  }

} // namespace hp1D
