/* Factored out part of building elements in hp2D::Space.
 */

#include "buildEdgeDofs.hh"
#include "quadFunctions.hh"

// debugging
#include "basics/debug.hh"

#define hpBuildEdgeDofsAppl_D 0
#define hpSpace2Ddof_D 0

namespace hp2D {

  // ********************************************************* BuildEdgeDofs **

  void BuildEdgeDofs::operator()(concepts::Quad& cntr,
				 concepts::TColumn<Real>*& T1) {
    conceptsAssert(spc_, concepts::Assertion());
    concepts::AdaptiveControlP<2>& ctrl2(ctrl2_(cntr.key()));
    concepts::TColumnTensor<Real, 2>* TTensor = 0;
    uint ll[2] = { 0, 0 };
    ushort Pmax[2] = { 0, 0 };
    computePmax_(cntr, Pmax);
    const uint nn[2] = { Pmax[0]+1, Pmax[1]+1 }; // p max

    // loop over edges
    concepts::Edge* cntr1 = 0;
    for(uint i = 0; (cntr1 = cntr.edge(i)) != 0; ++i) {
      concepts::AdaptiveControlP<1>& ctrl1(ctrl1_(cntr1->key()));
      DEBUGL(hpBuildEdgeDofsAppl_D, cntr.key() << " - " << *cntr1
	     << ", tag = " << ctrl1.tag_ << ", rho = " << cntr.rho(i)
	     << ", p = " << ctrl1.p_[0]);
      if (ctrl1.tag_.isMember() || ctrl1.tag_.isPassive()) {
	std::map<uint, concepts::EdgeData>::const_iterator j =
	  edgeListFind(cntr1->key().key());
	if (ctrl1.tag_.isMember() && ctrl1.tag_.isActive() &&
	    (j != edgeListEnd()) && (j->second.hasCell(cntr))) {
	  // active member
	  if (ctrl1.p_[0] > 1) {
	    // has dofs
	    if (! ctrl1.tag_.idxValid()) {
	      // not yet counted
	      ctrl1.idx_ = dim_(); dim_() += ctrl1.p_[0] - 1;
	      ctrl1.tag_.validateIdx();
	      DEBUGL(hpSpace2Ddof_D, *cntr1 << " has dof " << ctrl1.idx_);
	    }

	    // assume we have a quad
	    // get sign flip for the entry in the T matrix
	    // flip eventually sign for each second form function due to other
	    // direction of local axes : one side N(xi), other N(-xi)
	    conceptsAssert(i < 4, concepts::Assertion());
	    Real s = 0;
	    switch(i) {
	    case 0: case 1: s = cntr.rho(i) ? -1 : 1; break;
	    case 2: case 3: s = cntr.rho(i) ? 1 : -1; break;
	    default: throw conceptsException(concepts::ExceptionBase());
	    }
	    conceptsAssert3(s != 0, concepts::Assertion(), "s = " << s);

	    // get index information for the T matrix
	    uint pIndex = 0, qIndex = 0;
	    QuadFunctions::edgeOrientation(i, pIndex, qIndex);
	    QuadFunctions::edgeIndex(i, qIndex, ll);

	    Real t = 1.0;
	    uint idx = ctrl1.idx_;
	    for (int j = 0; j < ctrl1.p_[0] - 1; ++j) {
	      T1 = TTensor
		= new concepts::TColumnTensor<Real, 2>(nn, idx++, T1);
	      ll[pIndex] = j + 2;
	      DEBUGL(hpBuildEdgeDofsAppl_D, cntr.key()
		     << " - active dof: ll = (" << ll[0] << ", " << ll[1]
		     << "), edg = " << cntr1->key()
		     << ", idx = " << ctrl1.idx_+j << ", t = " << t);
	      (*TTensor)[ll] = t;
	      t *= s;
	    } // for j
	  } // p > 1
	} // active
	if (! ctrl2.tag_.isMember()) deactivate_(*cntr1);
      } // isMember || isPassive
      uint pIndex = 0, qIndex = 0;
      QuadFunctions::edgeOrientation(i, pIndex, qIndex);
    } // for i
  }

  BuildEdgeDofs* BuildEdgeDofs::clone(Space* spc) const {
    if (spc) return new BuildEdgeDofs(*spc);
    else return new BuildEdgeDofs(*spc_);
  }

} // namespace hp2D
