/* Factored out part of building elements in hp2D::Space.
 */

#include "buildInnerDofs.hh"
#include "quadFunctions.hh"

// debugging
#include "basics/debug.hh"

#define hpSpace2Ddof_D 0

namespace hp2D {

  // ******************************************************** BuildInnerDofs **

  void BuildInnerDofs::operator()(concepts::Quad& cntr,
				  concepts::TColumn<Real>*& T1) {
    conceptsAssert(spc_, concepts::Assertion());
    concepts::AdaptiveControlP<2>& ctrl2(ctrl2_(cntr.key()));
    concepts::TColumnTensor<Real, 2>* TTensor = 0;
    uint ll[2] = { 0, 0 };
    ushort Pmax[2] = { 0, 0 };
    computePmax_(cntr, Pmax);
    const uint nn[2] = { Pmax[0]+1, Pmax[1]+1 }; // p max

    if ( (ctrl2.p_[0] > 1) && (ctrl2.p_[1] > 1) ) { // p int
      ctrl2.idx_ = dim_();
      ctrl2.tag_.validateIdx();
      DEBUGL(hpSpace2Ddof_D, cntr << " has dof " << ctrl2.idx_);

      uint idx = ctrl2.idx_;
      for (uint& i = ll[0] = 2; i < (uint)ctrl2.p_[0]+1; ++i) { // p int
	for (uint& j = ll[1] = 2; j < (uint)ctrl2.p_[1]+1; ++j) { // p int
	  if (trunk(i, j, ctrl2.p_)) {
	    ++dim_();
	    T1 = TTensor
	      = new concepts::TColumnTensor<Real, 2>(nn, idx++, T1);
	    (*TTensor)[ll] = 1.0;
	  } // if "trunk"
	} // for j
      } // for i
    }
  }

  BuildInnerDofs* BuildInnerDofs::clone(Space* spc) const {
    if (spc) return new BuildInnerDofs(*spc);
    else return new BuildInnerDofs(*spc_);
  }

  bool BuildInnerDofs::trunk(uint p, uint q, const ushort P[2]) {
    DEBUGL(0, "p = " << p << ", q = " << q << ", returning true");
    return true;
  }

  // ************************************************ BuildInnerDofsLinTrunk **

 BuildInnerDofsLinTrunk* BuildInnerDofsLinTrunk::clone(Space* spc) const {
   if (spc) return new BuildInnerDofsLinTrunk(*spc);
   else return new BuildInnerDofsLinTrunk(*spc_);
 }

  bool BuildInnerDofsLinTrunk::trunk(uint p, uint q, const ushort P[2]) {
    DEBUGL(0, "p = " << p << ", q = " << q << ", P = (" << P[0] << ", " << P[1]
	   << "), returning " << ((p + q) <= ((P[0] > P[1]) ? P[0] : P[1]))
	   << " - " << (p + q) << " - " << ((P[0] > P[1]) ? P[0] : P[1]));
    return ((p + q) <= ((P[0] > P[1]) ? P[0] : P[1]));
  }

  // ************************************************ BuildInnerDofsHypTrunk **

 BuildInnerDofsHypTrunk* BuildInnerDofsHypTrunk::clone(Space* spc) const {
   if (spc) return new BuildInnerDofsHypTrunk(*spc);
   else return new BuildInnerDofsHypTrunk(*spc_);
 }

  bool BuildInnerDofsHypTrunk::trunk(uint p, uint q, const ushort P[2]) {
    const ushort Pmax = ((P[0] > P[1]) ? P[0] : P[1]);
    return (p*q <= 1.1*Pmax);
  }

} // namespace hp2D
