/* Factored out part of building elements in hp2D::Space.
 */

#include "buildVtxDofs.hh"
#include "quadFunctions.hh"

// debugging
#include "basics/debug.hh"

#define hpBuildVtxDofsAppl_D 0
#define hpSpace2Ddof_D 0

namespace hp2D {

  // ******************************************************* BuildVertexDofs **

  void BuildVertexDofs::operator()(concepts::Quad& cntr,
				   concepts::TColumn<Real>*& T1) {
    conceptsAssert(spc_, concepts::Assertion());
    concepts::AdaptiveControlP<2>& ctrl2(ctrl2_(cntr.key()));
    concepts::TColumnTensor<Real, 2>* TTensor = 0;
    uint ll[2] = { 0, 0 };
    ushort Pmax[2] = { 0, 0 };
    computePmax_(cntr, Pmax);
    const uint nn[2] = { Pmax[0]+1, Pmax[1]+1 }; // p max

    concepts::Vertex* cntr0 = 0;
    // loop over vertices
    for(uint i = 0; (cntr0 = cntr.vertex(i)) != 0; ++i) {
      concepts::AdaptiveControl& ctrl0(ctrl0_(cntr0->key()));
      DEBUGL(hpBuildVtxDofsAppl_D, cntr.key() << " - " << *cntr0
	     << ", tag = " << ctrl0.tag_);
      if (ctrl0.tag_.isMember() || ctrl0.tag_.isPassive()) {
	// is member of the space or active
	std::map<uint, concepts::VertexData>::const_iterator j =
	  vertexListFind(cntr0->key().key());
	if (ctrl0.tag_.isMember() && ctrl0.tag_.isActive() &&
	    (j != vertexListEnd()) && (j->second.hasCell(cntr))) {
	  // active member of the space
	  if (! ctrl0.tag_.idxValid()) { 
	    // not yet counted
	    ctrl0.idx_ = dim_(); ++dim_();
	    ctrl0.tag_.validateIdx();
	    DEBUGL(hpSpace2Ddof_D, *cntr0 << " has dof " << ctrl0.idx_);
	  }
	  T1 = TTensor
	    = new concepts::TColumnTensor<Real, 2>(nn, ctrl0.idx_, T1);

	  QuadFunctions::vertexIndex(i, ll);

 	  DEBUGL(hpBuildVtxDofsAppl_D, cntr.key()
		 << " - active dof: ll = (" << ll[0] << ", " << ll[1]
		 << "), vtx = " << cntr0->key() << ", idx = " << ctrl0.idx_);
	  (*TTensor)[ll] = 1.0;
	}
	// cell not 'member of space' deactivate all vertices (does nothing)
	if (! ctrl2.tag_.isMember()) deactivate_(*cntr0);
      }
    }
  }

  BuildVertexDofs* BuildVertexDofs::clone(Space* spc) const {
    if (spc) return new BuildVertexDofs(*spc);
    else return new BuildVertexDofs(*spc_);
  }

} // namespace hp2D
