/* Functions on space with hp elements in 2D
 */

#ifndef hpFunction2d_hh
#define hpFunction2d_hh

#include <cstring>
#include "space/function.hh"
#include "space/element.hh"
#include "quad.hh"

namespace hp2D {

  using concepts::Real;
  using concepts::Real2d;

  // ***************************************************************** Value **

  /** The approximated function in a FE space.
      @test test::FunctionEvaluation
      @author Kersten Schmidt, 2004
  */
  class Value : public concepts::ScalarElementFunction<Real> {
  public:
    virtual ~Value() {}
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     Real& val, const uint i[2]) const;
    virtual void operator() (const Quad& elm,
			     const concepts::Array<Real>& coeff,
			     Real& val, const uint i[2]) const;
    virtual void operator() (const concepts::Element<Real>& elm,
			     const uint j[2],
			     Real& val, const uint i[2]) const;
    void operator() (const Quad& elm, const uint j[2],
		     Real& val, const uint i[2]) const;
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     Real& val, const Real p,
			     const Real t = 0.0) const;
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     Real& val, const Real2d& p,
			     const Real t = 0.0) const;
    void operator() (const Quad& elm, const concepts::Array<Real>& coeff,
		     Real& val, const Real2d& p, const Real t = 0.0) const;
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     Real& val, const concepts::Real3d& p,
			     const Real t = 0.0) const;

  protected:
    virtual std::ostream& info(std::ostream& os) const;
  };

  // ****************************************************************** Grad **

  /** The gradient of the approximated function in a FE space.
      @author Kersten Schmidt, 2004
  */
  class Grad : public concepts::ElementFunction<Real> {
  public:
    virtual ~Grad() {}
    virtual uint n() const { return 2; }
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     concepts::Array<Real>& val,
			     const uint i[2]) const;
    virtual void operator() (const Quad& elm,
			     const concepts::Array<Real>& coeff,
			     concepts::Array<Real>& val,
			     const uint i[2]) const;
    virtual void operator() (const concepts::Element<Real>& elm,
			     const uint j[2],
			     concepts::Array<Real>& val,
			     const uint i[2]) const;
    virtual void operator() (const Quad& elm, const uint j[2],
			     concepts::Array<Real>& val,
			     const uint i[2]) const;
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     concepts::Array<Real>& val,
			     const Real p, const Real t = 0.0) const
      throw(concepts::MissingFeature) {
      throw conceptsException(concepts::MissingFeature("not implemented"));
    }
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     concepts::Array<Real>& val,
			     const Real2d& p, const Real t = 0.0) const
      throw(concepts::MissingFeature) {
      throw conceptsException(concepts::MissingFeature("not implemented"));
    }
    virtual void operator() (const concepts::Element<Real>& elm,
			     const concepts::Array<Real>& coeff,
			     concepts::Array<Real>& val,
			     const concepts::Real3d& p,
			     const Real t = 0.0) const
      throw(concepts::MissingFeature) {
      throw conceptsException(concepts::MissingFeature("not implemented"));
    }

  protected:
    virtual std::ostream& info(std::ostream& os) const;
  };

} // namespace hp2D

#endif // hpRealunction2d_hh
