/* shape function in 2D: data for shape values, abscissas and quadratur weights
 */

#ifndef shapeFucntion2d_hh
#define shapeFucntion2d_hh

#include "basics/typedefs.hh"
#include "basics/exceptions.hh"
#include "space/tmatrix.hh"

namespace hp2D {

  using concepts::Real;

  // ******************************************************* ShapeFunction2D **

  /** Collecting the data of a 2D shape function in one class.
      This class just collects all the data for a 2D shape function: it just
      holds pointers to the real data.
      @author Kersten Schmidt, 2003
  */
  template<typename F>
  class ShapeFunction2D {
  public:
    ShapeFunction2D();

    /// Returns number of shape functions
    uint n(uint i) const {
      conceptsAssert(i < 2, concepts::Assertion());
      conceptsAssert(n_[i] != 0, concepts::Assertion());
      return n_[i]; }
    /// Sets the number of shape functions
    void n(uint i, uint value) {
      conceptsAssert(i < 2, concepts::Assertion());
      n_[i] = value;
    }

    /// Returns number of quadratur points
    uint nq(uint i) const {
      conceptsAssert(i < 2, concepts::Assertion());
      conceptsAssert(nq_[i] != 0, concepts::Assertion());
      return nq_[i]; }
    /// Sets the number of quadratur points
    void nq(uint i, uint value) {
      conceptsAssert(i < 2, concepts::Assertion());
      nq_[i] = value;
    }

    /// Returns the values of the shape functions
    const Real* values(uint i) const {
      conceptsAssert(i < 2, concepts::Assertion());
      conceptsAssert(values_[i] != 0, concepts::Assertion());
      return values_[i]; }
    /// Sets the values of the shape functions
    void values(uint i, const Real* value) {
      conceptsAssert(i < 2, concepts::Assertion());
      values_[i] = value;
    }

    /// Returns the abscissas of the quadrature rule and the shape functions
    const Real* abscissas(uint i) const {
      conceptsAssert(i < 2, concepts::Assertion());
      conceptsAssert(abscissas_[i] != 0, concepts::Assertion());
      return abscissas_[i]; }
    /// Sets the abscissas of the quadrature rule and the shape functions
    void abscissas(uint i, const Real* value) {
      conceptsAssert(i < 2, concepts::Assertion());
      abscissas_[i] = value;
    }

    /// Returns the weights of the quadrature rule
    const Real* weights(uint i) const {
      conceptsAssert(i < 2, concepts::Assertion());
      conceptsAssert(weights_[i] != 0, concepts::Assertion());
      return weights_[i]; }
    /// Sets the weights of the quadrature rule
    void weights(uint i, const Real* value) {
      conceptsAssert(i < 2, concepts::Assertion());
      weights_[i] = value;
    }

    /// Returns the T matrix
    const concepts::TMatrixBase<F>* T() const {
      conceptsAssert(T_ != 0, concepts::Assertion());
      return T_; }
    /// Sets the T matrix
    void T(const concepts::TMatrixBase<F>* T) { T_ = T; }
  private:
    int n_[2], nq_[2];
    const F* values_[2];
    const Real* abscissas_[2], * weights_[2];
    const concepts::TMatrixBase<F>* T_;
  };

} // namespace hp2D

#endif // shapeFucntion2d_hh
