/* testsuite for 2D stiffness matrix for needle elements
 */

#include <iostream>
#include "vectorial/vectorial.hh"
#include "vectorial/vectorizable.hh"
#include "hp2D/rotrot.hh"
#include "hp2D/divdiv.hh"
#include "cloneing.hh"

namespace test {

  using concepts::Real;

  // ************************************************************** Cloneing **

  Cloneing::~Cloneing() {}

  void Cloneing::run() {
    vectorial::BilinearForm<Real> bfrot(2);
    hp2D::RotRot::setup(bfrot);
    testMaxwell<hp2D::RotRot>(bfrot);

    vectorial::BilinearForm<Real> bfdiv(2);
    hp2D::SingularSet sing;
    hp2D::DivDiv<hp2D::TrivialWeight>::setup(bfdiv, sing);
    testMaxwell<hp2D::DivDiv<hp2D::TrivialWeight> >(bfdiv);
  }

  template<typename max>
  void Cloneing::testMaxwell(vectorial::BilinearForm<Real>& bf) {
    for (uint i = 0; i < 4; ++i) {
      const max* cr =
	dynamic_cast<const max*>(bf.get(i));
      _test(cr != 0);
      max* r = const_cast<max*>(cr);
      _test(r != 0);
      _test(r->data() != 0);
    }

    std::auto_ptr<vectorial::BilinearForm<Real> > second(bf.clone());
    _test(second.get() != &bf);
    for (uint i = 0; i < 4; ++i) {
      const vectorial::Vectorizable* vc =
	dynamic_cast<const vectorial::Vectorizable*>(second->get(i));
      _test(vc != 0);
      vectorial::Vectorizable* v = const_cast<vectorial::Vectorizable*>(vc);
      _test(v->data() != 0);
    }

    concepts::Cloneable* secondData = 0, * firstData = 0;
    for (uint i = 0; i < 4; ++i) {
      _test(second->get(i) != bf.get(i));
      const vectorial::Vectorizable* vc =
	dynamic_cast<const vectorial::Vectorizable*>(bf.get(i));
      _test(vc != 0);
      vectorial::Vectorizable* v = const_cast<vectorial::Vectorizable*>(vc);
      _test(v != 0);
      if (firstData == 0)
	firstData = v->data();
      else
	_test(v->data() == firstData);
      vc = dynamic_cast<const vectorial::Vectorizable*>(second->get(i));
      _test(vc != 0);
      v = const_cast<vectorial::Vectorizable*>(vc);
      _test(v != 0);
      if (secondData == 0)
	secondData = v->data();
      else
	_test(v->data() == secondData);
    }
    _test(firstData != 0);
    _test(secondData != 0);
    _test(secondData != firstData);
  }

} // namespace test

int main(int, char**) {
  try {
    test::Cloneing t;
    t.run();
    return t.report();
  }
  catch (concepts::ExceptionBase& e) {
    std::cout << e << std::endl;
  }
  return 1;
}
