/** @file filename.cc Description
    @author Joe Foo, 2004
*/

#ifndef testname_hh
#define testname_hh

#include "basics/testcase.hh"
#include "basics/typedefs.hh"
#include "basics/vectorsMatricesForward.hh"

using concepts::Real;

namespace hp2D {
  class Space;
}

namespace test {

  // **************************************************** FunctionEvaluation **

  /** Tests the evaluation of functions like hp2D::Value. These
      function compute something out of a FE function.
      @author Philipp Frauenfelder, 2004
  */
  class FunctionEvaluation : public TestCase {
  public:
    virtual ~FunctionEvaluation() {}
    virtual void run();

    /// @name Test routines
    //@{
    /** Tests the evaluation of hp2D::Value at an arbitrary point in an
	element. This point does not need to conincide with an integration
	point.
    */
    void testValue(hp2D::Space& spc);
    /// Tests the evaluation of hp2D::Value at the integration points.
    void testValueInt(hp2D::Space& spc);
    //@}
  private:
    /// Computes shape function with index \c i in point \c p
    Real shapeFct_(const uint i[2], const concepts::Real2d& p) const;
  };

} // namespace test

#endif // hp2DFunctionEvalutation_hh
