/* weight for 2D transmission problem in (-1,1)^2 to the
   coordinate axis
*/

#include "transmissionWeight.hh"
#include "hp2D/shortestDist.hh"
#include "hp2D/singularSet.hh"
#include "hp2D/quad.hh"

namespace hp2D {

  // **************************************************** TransmissionWeight **

  concepts::Real TransmissionWeight::operator()(const hp2D::SingularSet&,
						concepts::Real2d point,
						const hp2D::Quad* elm) const {
    const concepts::Real2d p(elm->chi(point[0], point[1]));
    const concepts::Real x = p[0]*p[0], y = p[1]*p[1];
    return (x < y ? x : y);
  }

  std::ostream& operator<<(std::ostream& os, const TransmissionWeight& p) {
    return os << "TransmissionWeight";
  }

  // ************************************************ TransmissionWeightProd **

  concepts::Real TransmissionWeightProd::operator()
    (const hp2D::SingularSet&, concepts::Real2d point,
     const hp2D::Quad* elm) const {
    const concepts::Real2d p(elm->chi(point[0], point[1]));
    return p[0]*p[0] * p[1]*p[1];
  }

  std::ostream& operator<<(std::ostream& os, const TransmissionWeightProd& p) {
    return os << "TransmissionWeightProd";
  }

} // namespace hp2D
