/* weight for 2D transmission problem in (-1,1)^2 to the
   coordinate axis
*/

#ifndef transmissionWeight_hh
#define transmissionWeight_hh

#include "basics/vectorsMatrices.hh"

namespace hp2D {

  class SingularSet;
  class Quad;

  // **************************************************** TransmissionWeight **

  /** Weight for the 2D transmission problem. Computes the distance to the
      coordinate axis.
      @author Philipp Frauenfelder, 2004
      @ingroup maxwell_2d_wr
  */
  class TransmissionWeight {
  public:
    /// Returns the minimum of x<sup>2</sup> and y<sup>2</sup> of \c point
    concepts::Real operator()(const hp2D::SingularSet&, concepts::Real2d point,
			      const hp2D::Quad*) const;
  };

  std::ostream& operator<<(std::ostream& os, const TransmissionWeight& p);

  // ************************************************ TransmissionWeightProd **

  /** Weight for the 2D transmission problem. Computes the product of
      the distances to the coordinate axis.
      @author Philipp Frauenfelder, 2004
      @ingroup maxwell_2d_wr
  */
  class TransmissionWeightProd {
  public:
    /// Returns the product of x<sup>2</sup> and y<sup>2</sup> of \c point
    concepts::Real operator()(const hp2D::SingularSet&, concepts::Real2d point,
			      const hp2D::Quad*) const;
  };

  std::ostream& operator<<(std::ostream& os, const TransmissionWeightProd& p);

} // namespace hp2D

#endif // transmissionWeight_hh
