/* a priori refinement for hexahedra
 */

#include "aprioriRef.hh"
#include "hexahedron.hh"
#include "basics/exceptions.hh"
#include "geometry/topology.hh"

namespace hp3D {

  // ***************************************************** APrioriRefinement **

  void APrioriRefinement::operator() (const concepts::Element<Real>& elm)
    throw(concepts::MissingFeature) {

    const hp3D::Hexahedron* hex = dynamic_cast<const hp3D::Hexahedron*>(&elm);
    if (hex) {
      spc_.adjust(elm, computeRef_(hex->support()));
      return;
    } // if hex

    throw conceptsException(concepts::MissingFeature("element not supported"));
  }

  std::ostream& APrioriRefinement::info(std::ostream& os) const {
    return os << "hp3D::APrioriRefinement(vtx = " << attribVtx_ << ", edge = "
	      << attribEdge_ << ", face = " << attribFace_ << ", pmax = ("
	      << p_[0] << ", " << p_[1] << ", " << p_[2] << "))";
  }

  concepts::AdaptiveAdjustP<3> APrioriRefinement::computeRef_
  (const concepts::Hexahedron& h) const {
    concepts::Level<3> l(0);
    int p[3] = { p_[0], p_[1], p_[2] };

    if (attribVtx_) {
      concepts::Connector0* vtx = 0;
      for (uint i = 0; (vtx = h.vertex(i)) != 0; ++i) {
	if (vtx->attrib() == attribVtx_) {
	  l.l_[0] = l.l_[1] = l.l_[2] = 1;
	}
      } // for vertices
    }
    if (attribEdge_) {
      concepts::Connector1* edge = 0;
      for (uint i = 0;
	   (edge = h.edge(i)) != 0; ++i) {
	if (edge->attrib() == attribEdge_)
	  switch (i) {
	  case 4: case 5: case 6: case 7: l.l_[0] = 1; l.l_[1] = 1; break;
	  case 1: case 3: case 9: case 11: l.l_[0] = 1; l.l_[2] = 1; break;
	  case 0: case 2: case 8: case 10: l.l_[1] = 1; l.l_[2] = 1; break;
	  default:
	    throw conceptsException(concepts::ExceptionBase());
	  }
      } // for edges
    }
    if (attribFace_) {
      concepts::Connector2* face = 0;
      for (uint i = 0;
	   (face = h.face(i)) != 0; ++i) {
	if (face->attrib() == attribFace_)
	  switch (i) {
	  case 0: case 5: l.l_[2] = 1; break;
	  case 1: case 3: l.l_[1] = 1; break;
	  case 2: case 4: l.l_[0] = 1; break;
	  default:
	    throw conceptsException(concepts::ExceptionBase());
	  }
      } // for faces
    }

    for (uint i = 0; i < 3; ++i) {
      if (l.l_[i] != 0) p[i] = 0;
    }
    return concepts::AdaptiveAdjustP<3>(l, p);
  }

} // namespace hp3D
