/* Factored out part of building elements in hp3D::Space.
 */

#include "buildEdgeDofs.hh"
#include "hexFunctions.hh"

// debugging
#include "basics/debug.hh"

#define hpBuildEdgeDofsAppl_D 0
#define hpSpace3Ddof_D 0

namespace hp3D {

  // ********************************************************* BuildEdgeDofs **

  void BuildEdgeDofs::operator()(concepts::Hexahedron& cntr,
				   concepts::TColumn<Real>*& T1) {
    conceptsAssert(spc_, concepts::Assertion());
    const concepts::AdaptiveControlP<3>& ctrl3(ctrl3_(cntr.key()));
    concepts::TColumnTensor<Real, 3>* TTensor = 0;
    uint ll[3] = { 0, 0, 0 };
    ushort Pmax[3] = { 0, 0, 0 };
    computePmax_(cntr, Pmax);
    uint nn[3] = { Pmax[0]+1, Pmax[1]+1, Pmax[2]+1 };

    // loop over edges
    concepts::Edge* cntr1 = 0;
    for(uint i = 0; (cntr1 = cntr.edge(i)) != 0; ++i) {
      concepts::AdaptiveControlP<1>& ctrl1(ctrl1_(cntr1->key()));
      DEBUGL(hpBuildEdgeDofsAppl_D, cntr.key() << " - " << *cntr1
	     << ", tag = " << ctrl1.tag_);
      if (ctrl1.tag_.isMember() || ctrl1.tag_.isPassive()) {
	std::map<uint, concepts::EdgeData>::const_iterator j =
	  edgeListFind(cntr1->key().key());
	if (ctrl1.tag_.isMember() && ctrl1.tag_.isActive() &&
	    (j != edgeListEnd()) && (j->second.hasCell(cntr))) {
	  // active member
	  if (ctrl1.p_[0] > 1) {
	    // has dofs
	    if (! ctrl1.tag_.idxValid()) {
	      // not yet counted
	      ctrl1.idx_ = dim_(); dim_() += ctrl1.p_[0] - 1;
	      ctrl1.tag_.validateIdx();
	      DEBUGL(hpSpace3Ddof_D, *cntr1 << " has dof " << ctrl1.idx_);
	    }
	    Real s = 0.0;

	    // assume we have a hex
	    // get sign flip for the entry in the T matrix
	    conceptsAssert(i < 12, concepts::Assertion());
	    concepts::Vertex* vtx = cntr1->vertex(0);
	    switch (i) {
	    case 0: case 3: case 4:
	      if (cntr.vertex(0) == vtx) s = 1.0; else s = -1.0; break;
	    case 1: case 5:
	      if (cntr.vertex(1) == vtx) s = 1.0; else s = -1.0; break;
	    case 2: case 7:
	      if (cntr.vertex(3) == vtx) s = 1.0; else s = -1.0; break;
	    case 6:
	      if (cntr.vertex(2) == vtx) s = 1.0; else s = -1.0; break;
	    case 8: case 11:
	      if (cntr.vertex(4) == vtx) s = 1.0; else s = -1.0; break;
	    case 9:
	      if (cntr.vertex(5) == vtx) s = 1.0; else s = -1.0; break;
	    case 10:
	      if (cntr.vertex(7) == vtx) s = 1.0; else s = -1.0; break;
	    default: throw conceptsException(concepts::ExceptionBase());
	    }

	    // get index information for the T matrix
	    uint pIndex = 0, qIndex = 0, rIndex = 0;
	    HexFunctions::edgeOrientation(i, pIndex, qIndex, rIndex);
	    HexFunctions::edgeIndex(i, qIndex, rIndex, ll);
	    conceptsAssert(pIndex != rIndex, concepts::Assertion());
	    conceptsAssert(qIndex != rIndex, concepts::Assertion());
	    conceptsAssert(pIndex != qIndex, concepts::Assertion());

	    Real t = 1.0;
	    uint idx = ctrl1.idx_;
 	    for (int j = 0; j < ctrl1.p_[0] - 1; ++j) {
	      T1 = TTensor
		= new concepts::TColumnTensor<Real, 3>(nn, idx++, T1);
	      ll[pIndex] = j + 2;
	      (*TTensor)[ll] = t;
	      t *= s;
	      DEBUGL(hpBuildEdgeDofsAppl_D, cntr.key()
		     << " - active dof: ll = (" << ll[0] << ", " << ll[1]
		     << ", " << ll[2] << "), p = " << ctrl1.p_[0]
		     << ", idx = " << ctrl1.idx_+j);
	    } // for j
	  } // p > 1
	} // active
	if (! ctrl3.tag_.isMember()) deactivate_(*cntr1);
      }

      uint pIndex = 0, qIndex = 0, rIndex = 0;
      HexFunctions::edgeOrientation(i, pIndex, qIndex, rIndex);
    } // for i
  }

  BuildEdgeDofs* BuildEdgeDofs::clone(Space* spc) const {
    if (spc) return new BuildEdgeDofs(*spc);
    else return new BuildEdgeDofs(*spc_);
  }

} // namespace hp3D
