// additional functionality for hexahedra

#include "hexFunctions.hh"
#include "basics/exceptions.hh"
#include "geometry/cell3D.hh"

namespace hp3D {

  // ********************************************************** HexFunctions **

  void HexFunctions::edgeOrientation(const uint i, uint& pIndex,
				     uint& qIndex, uint& rIndex) {
    conceptsAssert(i < 12, concepts::Assertion());
    switch(i) {
    case 0: case 2: case 8: case 10: pIndex = 0; qIndex = 1; rIndex = 2; break;
    case 1: case 3: case 9: case 11: pIndex = 1; qIndex = 0; rIndex = 2; break;
    case 4: case 5: case 6: case 7: pIndex = 2; qIndex = 0; rIndex = 1; break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
  }

  void HexFunctions::faceOrientation(const concepts::Hexahedron& cntr,
				     const uint i, uint& pIndex,
				     uint& qIndex, uint& rIndex) {
    conceptsAssert(i < 6, concepts::Assertion());
    switch(i) {
    case 0: case 5: pIndex = 0; qIndex = 1; break;
    case 1: case 3: pIndex = 2; qIndex = 0; break;
    case 2: case 4: pIndex = 2; qIndex = 1; break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
    if (cntr.tau(i) % 2) std::swap(pIndex, qIndex);
    switch(i) {
    case 0: case 5: rIndex = 2; break;
    case 2: case 4: rIndex = 0; break;
    case 1: case 3: rIndex = 1; break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
  }

  void HexFunctions::faceShapefctOrientation(const concepts::Hexahedron& cntr,
					     const uint i,
					     bool& flip0, bool& flip1) {
    // rho = 0, tau = 0
    switch(i) {
    case 0: flip0 = false; flip1 = false; break;
    case 1: flip0 = true; flip1 = false; break;
    case 2: flip0 = true; flip1 = false; break;
    case 3: flip0 = true; flip1 = true; break;
    case 4: flip0 = true; flip1 = true; break;
    case 5: flip0 = true; flip1 = false; break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
    // rho = 1
    if (cntr.rho(i)) flip0 = !flip0;
    // tau /= 0
    for (int j = 0; j < cntr.tau(i); ++j) {
      std::swap(flip0, flip1);
      flip0 = !flip0;
    }
  }

  void HexFunctions::vertexIndex(const uint i, uint ll[3]) {
    uint m = i;
    if (m > 3) {
      m -= 4;
      ll[2] = 1;
    } else
      ll[2] = 0;
    switch(m) {
    case 0: ll[0] = 0; ll[1] = 0; break;
    case 1: ll[0] = 1; ll[1] = 0; break;
    case 2: ll[0] = 1; ll[1] = 1; break;
    case 3: ll[0] = 0; ll[1] = 1; break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
  }

  void HexFunctions::edgeIndex(const uint i,
			       const uint qIndex, const uint rIndex,
			       uint ll[3]) {
    switch (i) {
    case  0: case  3: case  4: ll[qIndex] = 0; ll[rIndex] = 0; break;
    case  1: case  2: case  5: ll[qIndex] = 1; ll[rIndex] = 0; break;
    case  7: case  8: case 11: ll[qIndex] = 0; ll[rIndex] = 1; break;
    case  6: case  9: case 10: ll[qIndex] = 1; ll[rIndex] = 1; break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
  }

  void HexFunctions::faceIndex(const uint i, const uint rIndex, uint ll[3]) {
    switch(i) {
    case 0: case 1: case 4: ll[rIndex] = 0; break;
    case 2: case 3: case 5: ll[rIndex] = 1; break;
    default: throw conceptsException(concepts::ExceptionBase());
    }
  }

} // namespace hp3D
