// additional functionality for hexahedra

#ifndef hexFunctions_hh
#define hexFunctions_hh

#include "basics/typedefs.hh"

namespace concepts {

  // forward declaration
  class Hexahedron;

}

namespace hp3D {

  // ********************************************************** HexFunctions **

  /** Auxiliary functions for hexahedra.
      @author Philipp Frauenfelder, 2002
  */
  class HexFunctions {
  public:
    /** Helps distributing the polynomial degree to the edges
	@param i Index of the edge [0..11]
	@param pIndex Index of the polynomial degree
	              of this edge in the array of the element
	@param qIndex,rIndex Other indices
	@return pIndex, qIndex, rIndex
    */
    static void edgeOrientation(const uint i,
				uint& pIndex, uint& qIndex, uint& rIndex);

    /** Helps distributing the polynomial degree to the faces
	@param cntr Connector
	@param i Index of the face [0..5]
	@param pIndex Index of the first polynomial degree
	              of this face in the array of the element
	@param qIndex Index of the second polynomial degree
	              of this face in the array of the element
	@param rIndex Other index
	@return pIndex, qIndex, rIndex
    */
    static void faceOrientation(const concepts::Hexahedron& cntr, const uint i,
				uint& pIndex, uint& qIndex, uint& rIndex);

    /** Determines wether a shape function direction has to flip the
	sign for the odd degree polynomials.
	@param cntr Connector
	@param i Index of the face [0..5]
	@param flip0 Return value, the first direction has to flip
	@param flip1 Return value, the second direction has to flip
    */
    static void faceShapefctOrientation(const concepts::Hexahedron& cntr,
					const uint i,
					bool& flip0, bool& flip1);

    /** Computes the tensor product index \c ll for a vertex with index \c i.
	@return ll
    */
    static void vertexIndex(const uint i, uint ll[3]);

    /** Computes the tensor product index \c ll for an edge with index \c i
	and precomputed indices \c qIndex and \c rIndex (use edgeOrientation to
	compute them).
	@return ll
    */
    static void edgeIndex(const uint i, const uint qIndex, const uint rIndex,
			  uint ll[3]);

    /** Computes the tensor product index \c ll for a face with index \c i
	and precomputed index \c rIndex (use faceOrientation to compute it).
	@return ll
    */
    static void faceIndex(const uint i, const uint rIndex, uint ll[3]);

    /** Computes the linear index from the tensor product index \c ll using
	the polynomial degrees \c p.
	@return Linear index
    */
    static uint index(const uint ll[3], const ushort p[3]) {
      // copy'n paste from TColumnTensor::operator[]
      return ll[0]+ll[1]*(1+p[0])+ll[2]*(1+p[0])*(1+p[1]);
    }
  };

} // namespace hp3D

#endif // hexFunctions_hh
