/** @file hp3D/maxwell.hh Shared data for the Maxwell bilinear forms DivDiv
    and RotRot in three dimensions.
*/

#ifndef maxwell3D_hh
#define maxwell3D_hh

#include "basics/cloneable.hh"
#include "toolbox/array.hh"
#include "basics/vectorsMatrices.hh"

namespace hp3D {
  // forward declarations
  class Hexahedron;

  using concepts::Real;

  // ***************************************************** MaxwellSharedData **

  /** Shared data for RotRot and DivDiv. The different components of
      the vector valued bilinear form for the rot-rot or div-div term
      need the Jacobian of the element map in exactly the same
      points. This is taken care of by the data fields of this
      class. They are common to all components of the vector valued
      bilinear form.

      RotRot::setup and DivDiv::setup take care of setting all up
      correctly. It uses the function
      RotRot::data(concepts::Cloneable*) and
      DivDiv::data(concepts::Cloneable*) to set the shared data of the
      components.
      @author Philipp Frauenfelder, 2004
      @ingroup maxwell_3d_wr
  */
  class MaxwellSharedData : public concepts::Cloneable {
  public:
    MaxwellSharedData() : jacobian(0), jacobianInv(0), elm(0) {}
    virtual ~MaxwellSharedData() {}
    virtual MaxwellSharedData* clone() const {
      return new MaxwellSharedData(); }
    concepts::Array<Real> jacobian;
    concepts::Array<concepts::MapReal3d> jacobianInv;
    const Hexahedron* elm;
  };

} // namespace hp3D

#endif // maxwell3D_hh
