// transition from one to another space

#ifndef spaceTransition_hh
#define spaceTransition_hh

#include "operator/compositions.hh"
#include "space/elementPairs.hh"
#include "space.hh"

namespace hp3D {

  // ******************************************************** TransitionPair **

  /** Element pair for the \c SpaceTransition.
      @author Philipp Frauenfelder, 2003
  */
  class TransitionPair : public concepts::ElementPair<Real> {
  public:
    TransitionPair(const Element<Real>& elm1, const Element<Real>& elm2,
		   const int relation) :
      elm1_(elm1), elm2_(elm2), relation_(relation) {}
    virtual const Element<Real>& elm1() const { return elm1_; }
    virtual const Element<Real>& elm2() const { return elm2_; }
    /// Returns the relation of the two elements
    int relation() const { return relation_; }
  private:
    const Element<Real>& elm1_, & elm2_;
    const int relation_;
  };

  // ******************************************************* SpaceTransition **

  /** Maps a solution vector from one space to another. Works very well
      for most apriori meshes.
      @bug If used for randomized meshes, some basis functions are not mapped
           exactly.
      @author Philipp Frauenfelder, 2003
  */
  class SpaceTransition : public concepts::Operator<Real> {
  public:
    /** Constructor. A list of \c ElementPairs is constructed.
	@param spaceX
	@param spaceY
    */
    SpaceTransition(const Space& spaceX, const Space& spaceY);
    virtual void operator()(const concepts::Function<Real>& fncY,
			    concepts::Function<Real>& fncX);
    void operator()(const concepts::Vector<Real>& fncY,
		    concepts::Vector<Real>& fncX);
    virtual const Space& spaceX() const { return spcX_; }
    virtual const Space& spaceY() const { return spcY_; }
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// The spaces
    const Space& spcX_, & spcY_;
    /// List of element pairs
    concepts::ElementPairList<Real> elmPairs_;
    /** Finds a ptah from \c large to \c small. The indices of the
	children which have to be taken are stored in \c children. If
	a hit is found, \c true is returned.
	@param level Level difference between \c large and \c small
	@param large Large cell
	@param small Small cell (is a descendant of \c large)
	@param children Array to store the path in
    */
    bool findChildPath_(const int level, const concepts::Hexahedron3d& large,
			const concepts::Hexahedron3d& small,
			concepts::Array<uint>& children);
  };

} // namespace hp3D

#endif // spaceTransition_hh
