#ifndef dgMeshInfo_hh
#define dgMeshInfo_hh

#include "edgeInfo.hh"

// forward declaration
namespace concepts {
  class Mesh2;
}

namespace linDG2D {

  /** Mesh info about edges for DG FEM in 2D. Scans a mesh for
      all edges and builds up EdgeInfo objects for all edges efficiently.
      The EdgeInfo objects can be scanned over or accessed through the Edge
      object itself. The MeshInfo object serves as a common base to build
      several DG spaces on the same mesh.
      @author Christoph Winkelmann, 2004
  */
  class MeshInfo {
  public:
    typedef concepts::Scan<EdgeInfo> Scanner;
    /** Constructor.
        @param mesh the mesh to build information up for
    */
    MeshInfo(concepts::Mesh2& mesh);
    /// scanner over EdgeInfo objects
    Scanner* scan() const { return new S(edgeInfos_); }
    /// Returns the edge info object built up for edge \c edge
    const EdgeInfo& edgeInfo(const concepts::Edge& edge) const {
      return edgeInfos_.find(edge.key())->second;
    }
  private:
    /// Scanner class for MeshInfo
    class S : public Scanner {
    public:
      inline S(const __gnu_cxx::hash_map<uint, EdgeInfo>& edgeInfos) :
        idx_(edgeInfos.begin()), edgeInfos_(edgeInfos) {}
      inline S(const S& scan) : idx_(scan.idx_), edgeInfos_(scan.edgeInfos_) {}
      inline bool eos() const { return idx_ == edgeInfos_.end(); }
      inline EdgeInfo& operator++(int) {
        return const_cast<EdgeInfo&>((*idx_++).second);
      }
      inline S* clone() const { return new S(*this); }
    private:
      __gnu_cxx::hash_map<uint, EdgeInfo>::const_iterator idx_;
      const __gnu_cxx::hash_map<uint, EdgeInfo>& edgeInfos_;
    };
    /// List of edge information objects
    __gnu_cxx::hash_map<uint, EdgeInfo> edgeInfos_;
  };

} // namespace linDG2D

#endif // dgMeshInfo_hh
