// element for linear FEM in 1D

#include "element1D.hh"
#include "lineGraphics.hh"
#include "basics/exceptions.hh"
#include "toolbox/elementGraphics.hh"

namespace linearFEM {

  // ****************************************************************** Line **

  std::auto_ptr<LineGraphics> Line::graphics_(0);

  Real Line::shapefct(const uint i, const Real xi) const {
    conceptsAssert(xi >= -1.0, concepts::Assertion());
    conceptsAssert(xi <= 1.0, concepts::Assertion());
    switch(i) {
    case 0: return (1.0+xi)/2.0;
    case 1: return (1.0-xi)/2.0;
    default: conceptsAssert(false, concepts::Assertion()); return 0.0;
    }
  }

  Real Line::shapefctD(const uint i) const {
    switch(i) {
    case 0: return 1.0/2.0;
    case 1: return -1.0/2.0;
    default: conceptsAssert(false, concepts::Assertion()); return 0.0;
    }
  }

  const concepts::ElementGraphics<Real>* Line::graphics() const {
    if (! graphics_.get())
      graphics_.reset(new LineGraphics());
    return graphics_.get();
  }

  std::ostream& Line::info(std::ostream& os) const {
    return os << "Line(" << cell_ << ", T = " << *T_ << ')';
  }

} // namespace linearFEM
