// element for linear FEM in 3D

#ifndef element3DLin_hh
#define element3DLin_hh

#include "element.hh"
#include "geometry/cell3D.hh"

namespace concepts {
  // forward declaration
  template<typename F>
  class ElementGraphics;
}

namespace linearFEM {
  // forward declaration
  class TetrahedronGraphics;

  // *********************************************************** Tetrahedron **

  /** Tetrehedral element with linear shape functions in 2D.
      @author Philipp Frauenfelder, 2002
  */
  class Tetrahedron : public Element {
  public:
    /** Constructor.
	@param cell Cell of the element
	@param idx Assembling information
    */
    Tetrahedron(const concepts::Tetrahedron3d& cell, uint idx[]) :
      Element(4, idx), cell_(cell) {}
    /** Constructor.
	@param cell Cell of the element
	@param T0 Assembling information
	@param T1 Assembling information
    */
    Tetrahedron(const concepts::Tetrahedron3d& cell,
		concepts::TColumn<Real>* T0,
		concepts::TColumn<Real>* T1 = 0) :
      Element(T0, T1), cell_(cell) {}
    virtual const concepts::Tetrahedron& support() const
    { return cell_.connector(); }
    virtual const concepts::Tetrahedron3d& cell() const { return cell_; }
    virtual const concepts::ElementGraphics<Real>* graphics() const;
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// Cell of the element
    const concepts::Tetrahedron3d& cell_;
    /// Graphics object of the element
    static std::auto_ptr<TetrahedronGraphics> graphics_;
  };

} // namespace linearFEM

#endif // element3DLin_hh
