// space for linear FEM in 3D

#ifndef linSpace3D_hh
#define linSpace3D_hh

#include "element3D.hh"
#include "basics/typedefs.hh"
#include "space/space.hh"
#include "space/element.hh"
#include "toolbox/scannerConnectors.hh"
#include "toolbox/dynArray.hh"

namespace concepts {

  // forward declarations
  class Mesh3;
  class BoundaryConditions;

  // ************************************************************** Scanners **

  template<>
  class Scan<linearFEM::Tetrahedron> : public Scan<linearFEM::Element> {
  public:
    virtual linearFEM::Tetrahedron& operator++(int) = 0;
  };

} // namespace concepts

namespace linearFEM {

  // ************************************************************** Linear3d **

  /** Space for linear FEM in 3D using tetrahedra.
      @author Philipp Frauenfelder, 2002
  */
  class Linear3d : public concepts::Space<Real> {
  public:
    typedef concepts::Scan<Tetrahedron> Scan;
    typedef void (*SMap)(const concepts::TColumn<Real>&,
			 concepts::TColumn<Real>&);
    /** Constructor.
	@param msh Mesh
	@param bc Boundary conditions
    */
    Linear3d(concepts::Mesh3& msh, concepts::BoundaryConditions* bc = 0);
    virtual ~Linear3d();
    virtual uint dim() const { return dim_; }
    virtual uint nelm() const { return nelm_; }
    virtual Scan* scan() const {
      return new concepts::PListScan<Tetrahedron>(*elm_); }
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// Number of degrees of freedom
    uint dim_;
    /// Number of elements
    uint nelm_;
    /// Boundary conditions
    concepts::BoundaryConditions* bc_;
    /// List of elements
    concepts::Joiner<Tetrahedron*, 1>* elm_;
  };

} // namespace linearFEM

#endif // linSpace3D_hh
