/* Bilinear Form
 */

#include "basics/typedefs.hh"
#include "bilinearForm.hh"
#include "space/element.hh"
#include "space/tmatrix.hh"

namespace concepts {

  // ********************************************************** BilinearForm **

  template<class F>
  std::ostream& BilinearForm<F>::info(std::ostream& os) const {
    return os << "BilinearForm()";
  }

  template class BilinearForm<Real>;

  // ****************************************************** BilinearFormLiCo **

  template<class F>
  BilinearFormLiCo<F>::BilinearFormLiCo(const BilinearFormLiCo& b) :
    bfAptr_((BilinearFormLiCo<F>*)b.bfA_->clone()),
    bfBptr_((BilinearFormLiCo<F>*)b.bfB_->clone()),
    bfA_(bfAptr_.get()), bfB_(bfBptr_.get()), cA_(b.cA_), cB_(b.cB_), em_() {}

  template<class F>
  void BilinearFormLiCo<F>::operator()(const Element<F>& elmX,
				       const Element<F>& elmY, 
				       ElementMatrix<F>& em) {
    em_.resize(elmX.T().m(), elmY.T().m());
    em_.zeros();
    (*bfA_)(elmX, elmY, em_);
    (*bfB_)(elmX, elmY, em);
    F* emData = (F*)em;
    F* em_Data = (F*)em_;
    for (uint i = 0; i < elmX.T().m()*elmY.T().m(); ++i, emData++, em_Data++) {
      *emData = cA_ * *em_Data + cB_ * *emData;
    }
  }

  template<class F>
  BilinearFormLiCo<F>* BilinearFormLiCo<F>::clone() const {
    return new BilinearFormLiCo<F>(*this);
  }

  template<class F>
  std::ostream& BilinearFormLiCo<F>::info(std::ostream& os) const {
    return os << "BilinearFormLiCo(" << cA_ << '*' << bfA_ << " + "
	      << cB_ << '*' << bfB_ << ')';
  }

  template class BilinearFormLiCo<Real>;

} // namespace concepts
