/* Basic operator declarations such as compositions of operators
 */

#include "compositions.cc"

#if defined(__hpux) || defined(_IBMR2)
  #define FORTRAN(X) X
#else
  #define FORTRAN(X) X ## _
#endif

#ifdef __cplusplus
extern "C" {
#endif
  extern void FORTRAN(dgemm)(const char& TRANSA, const char& TRANSB, 
			     const int& M, const int& N, const int& K, 
			     const double& ALPHA, 
			     const double* const A, const int& LDA,
			     const double* const B, const int& LDB, 
			     const double& BETA, const double* C,
			     const int& LDC);
#ifdef __cplusplus
}
#endif

namespace concepts {

  // ************************************************************** Operator **

  template class Operator<Real>;

  // *************************************************************** Compose **

  template<>
  void Compose<Real>::collapse_(DenseMatrix<Real>& A, DenseMatrix<Real>& B,
				Matrix<Real>& dest) const {
    // action with dgemm_
    const int m = B.spaceY().dim();
    const int n = A.spaceX().dim();
    Array<Real> res(m*n);
    const int k = B.spaceX().dim();
    const double alpha = 1.0;
    const int lda = B.spaceY().dim();
    const double beta = 0.0;
    const int ldb = A.spaceY().dim();
    const int ldc = B.spaceY().dim();
    DEBUGL(ComposeCollapse_D, "calling dgemm with parameters " 
	   << 'N' << ", " << 'N' << ", " << m << ", " << n << ", " << k 
	   << ", " << alpha << ", " << (double*)B << ", " << lda
	   << ", " << (double*)A << ", " << ldb << ", " << beta
	   << ", " << dest << ", " << ldc);
    FORTRAN(dgemm)('N', 'N', m, n, k, alpha, (double*)B, lda, 
		   (double*)A, ldb, beta, (Real*)res, ldc);
    DEBUGL(ComposeCollapse_D, "res = " << res);
    const Real* a = (Real*)res;
    for (uint i = 0; i < (uint)n; ++i)
      for (uint j = 0; j < (uint)m; ++j, ++a)
	if (*a != 0.0) dest(i,j) = *a;
  }

  template class Compose<Real>;

  // ***************************************************************** LiCoI **

  template class LiCoI<Real>;

  // ****************************************************************** LiCo **

  template class LiCo<Real>;

} // namespace concepts
