// solves a constrained source problem

#ifndef constrainedSP_hh
#define constrainedSP_hh

#include <memory>
#include "compositions.hh"
#include "sparseMatrix.hh"

namespace concepts {

  // forward declaration
  template<typename F>
  class SolverFabric;

  // *********************************************************** Constrained **

  /** Solves a linear system of equations subject to linear constraints.

      @author Philipp Frauenfelder, 2004
      @see Ainsworth, <em>Essential boundary conditions and multi-point
           constraints in finite element analysis</em>, 2000,
	   Comput. Methods Appl Mech. Engrg. 190 (2001) <a href="http://dx.doi.org/10.1016/S0045-7825(01)00236-5">6323-6339</a>
      @see Namespace constraints
      @ingroup solver
  */
  class Constrained : public Operator<Real> {
  public:
    /** Constructor.
	@param S Stiffness matrix
	@param C Constraints matrix
	@param g Right hand side of the constraints
    */
    Constrained(SparseMatrix<Real>& S, SparseMatrix<Real>& C,
		const Vector<Real>& g, SolverFabric<Real>& fabric);
    virtual void operator()(const Function<Real>& fncY, Function<Real>& fncX);
    virtual const Space<Real>& spaceX() const;
    virtual const Space<Real>& spaceY() const;
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    SparseMatrix<Real>& C_, &S_;
    SparseMatrix<Real> Ct_;
    std::auto_ptr<SparseMatrix<Real> > CCt_;
    const Vector<Real>& g_;
    Vector<Real> SRg_;
    std::auto_ptr<Operator<Real> > CCtinv_, solver_;
    std::auto_ptr<concepts::Compose<Real> > R_, Qmin_, CtC_, QS_, QSQ_;
    std::auto_ptr<LiCoI<Real> > Q_;
    std::auto_ptr<LiCo<Real> > Sp_;
    SolverFabric<Real>& fabric_;
  };

} // namespace concepts

#endif // constrainedSP_hh
