/* Iterative linear solver: GMRes
 */

#include <sstream>
#include "gmres.hh"

// #include "basics/debug.hh"

namespace concepts {

  // ***************************************************************** GMRes **

  template<class F>
  GMRes<F>::~GMRes() {}

  template<class F>
  void GMRes<F>::operator()(const Function<F>& fncY, Function<F>& fncX)
    throw(NoConvergence, MissingFeature) {

    const Vector<F>* fncYA;
    if ((fncYA = dynamic_cast<const Vector<F>*>(&fncY))) {
      Vector<F>* fncXA;
      if ((fncXA = dynamic_cast<Vector<F>*>(&fncX))) {
	(*this)(*fncYA, *fncXA); return;
      }
    }

    throw conceptsException(MissingFeature("function not supported"));
  }

  template<class F>
  std::ostream& GMRes<F>::info(std::ostream& os) const {
    return os << "GMRes(solves " << A_ << ", eps = " << eps_
	      << ", it = " << it_ << ((W_ != 0) ? ", with " : ", without ")
	      << "precond.)";
  }

} // namespace concepts
