// fabric class for GMRes

#ifndef GMResfabric_hh
#define GMResfabric_hh

#if __GNUC__ == 2
#  include <float.h>
#  define EPS DBL_EPSILON
#else
#  include <limits>
#  define EPS std::numeric_limits<double>::epsilon()
#endif
#include "operator/gmres.hh"
#include "operator/solverFabric.hh"

namespace concepts {

  // *********************************************************** GMResFabric **

  /** Fabric class for generalized minimal residual: \c GMRes.
      @author Christoph Winkelmann, 2004
  */
  template<typename F>
  class GMResFabric : public SolverFabric<F> {
  public:
    /** Constructor.
        @param maxeps Maximal residual
        @param maxit Maximal number of iterations
        @param rs Restart
        @param relres 0: absolute residual, 1: relative residual
    */
    GMResFabric(Real maxeps = EPS,
                int maxit = 0, uint rs = 0, uint relres = 0)
      : maxeps_(maxeps), maxit_(maxit), rs_(rs), relres_(relres) {}
    virtual Operator<F>* operator()(Operator<F>& matrix) {
      return new GMRes<F>(matrix, maxeps_, maxit_, rs_, relres_);
    }
  protected:
    virtual std::ostream& info(std::ostream& os) const {
      return os << "GMResFabric(maxeps = " << maxeps_ << ", maxit = "
                << maxit_ << ", rs = " << rs_ << ", relres = " 
                << relres_ << ")";
    }
  private:
    /// Convergence criteria for the solver
    Real maxeps_;
    /// Convergence criteria for the solver
    int maxit_;
    /// Restart
    uint rs_;
    /// relres = 0: absolute residual, relres = 1: relative residual
    uint relres_;
  };

} // concepts

#endif // GMResfabric_hh
