/* SparseMatrix
 * sparse matrices
 */

#include "hashedSMatrix.cc"
#include "basics/typedefs.hh"

// debugging
#include "basics/debug.hh"

#define HSMcompress_D 0

namespace concepts {

  template<>
  void HashedSparseMatrix<Cmplx>::compress(Real threshold) {
    DEBUGL(HSMcompress_D, "threshold = " << threshold);
    // search largest entry by magnitude
    Real maxValue = 0;
    uint hashBits = HashBits();
    uint p = 1 << hashBits;
    uint n = nofRows;
    HashedSparseMatrix<Cmplx>::Value** const matrix = Matrix();

    for (uint r = 0; r < n; ++r) {
      for (uint c_mod = 0; c_mod < p; ++c_mod) {
	for (HashedSparseMatrix<Cmplx>::Value* v =
	       matrix[(r<<hashBits) + c_mod]; v != NULL; v = v->lnk) {
	  Real abs = std::abs(v->val);
	  if (abs > maxValue) maxValue = abs;
	}
      }
    }
    DEBUGL(HSMcompress_D, "maxValue = " << maxValue);
    maxValue *= threshold;
    DEBUGL(HSMcompress_D, "drop entries below " << maxValue);

    for (uint r = 0; r < n; ++r) {
      for (uint c_mod = 0; c_mod < p; ++c_mod) {
	Value* vPrev = 0;
	for (HashedSparseMatrix<Cmplx>::Value* v =
	       matrix[(r<<hashBits) + c_mod]; v != NULL; v = v->lnk) {
	  if (std::abs(v->val) < maxValue) {
	    DEBUGL(HSMcompress_D, "dropping (" << r << ", " << v->idx
		   << ") = " << v->val);
	    dropEntry_(v, vPrev, r, v->idx);
	  } else
	    vPrev = v;
	}
      }
    }
  }

  template class HashedSparseMatrix<Cmplx>;
}
