// fabric class for Pardiso

#ifndef pardisoFabric_hh
#define pardisoFabric_hh

#include "pardiso.hh"
#include "solverFabric.hh"

namespace concepts {

  // ********************************************************* PardisoFabric **

  /** Fabric class for \c Pardiso.
      @author Philipp Frauenfelder, 2002
  */
  class PardisoFabric : public SolverFabric<Real> {
  public:
    PardisoFabric(enum Pardiso::MATRIX_TYPE type) : type_(type) {}
    virtual Operator<Real>* operator()(Operator<Real>& matrix) {
      return new Pardiso(matrix, type_);
    }
    virtual Operator<Real>* operator()(SparseMatrix<Real>& matrix) {
      return new Pardiso(matrix, type_);
    }
  protected:
    virtual std::ostream& info(std::ostream& os) const {
      return os << "PardisoFabric(" << type_ << ')';
    }
  private:
    enum Pardiso::MATRIX_TYPE type_;
  };

} // concepts

#endif // pardisoFabric_hh
