// interface to SPOOLES

#ifndef SPOOLES_HH
#define SPOOLES_HH

#include "basics/typedefs.hh"
#include "compositions.hh"

namespace concepts {

  using concepts::Real;

  // forwards
  template<typename F>
  class SparseMatrix;

  // *************************************************************** Spooles **

  /** Sparse direct solver for symmetric and unsymmetric matrices.
      @see <a href="http://www.netlib.org/linalg/spooles/spooles.2.2.html">SPOOLES Homepage</a>
      @author Philipp Frauenfelder, 2003
   */
  class Spooles : public Operator<Real> {
  public:
    /** Constructor.
	@param A Matrix
	@param symmetric Set this to true if the matrix is symmetric
    */
    Spooles(const SparseMatrix<Real>& A, bool symmetric = false);
    virtual ~Spooles();
    virtual void operator()(const Function<Real>& fncY, Function<Real>& fncX);
    void operator()(const Vector<Real>& fncY, Vector<Real>& fncX);
    virtual const Space<Real>& spaceX() const { return spcX_; }
    virtual const Space<Real>& spaceY() const { return spcY_; }
    uint iterations() const { return 0; }
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  private:
    /// System size
    const uint n_;
    /// Image space
    const Space<Real>& spcX_;
    /// Source space
    const Space<Real>& spcY_;
    /// Symmetric system?
    bool symmetric_;
    /// Stores if the factorization was already computed before
    bool factored_;

    /// SPOOLES data
    void* mtxA, * frontmtx, * mtxmanager, * chvmanager, * rootchv,
      * frontETree, * graph, * newToOldIV, * oldToNewIV, * adjIVL,
      * symbfacIVL, * bridge;
    int* oldToNew, * newToOld;
    int rc;
  };

} // namespace concepts

#endif // SPOOLES_HH
