/* testsuite for NRLsolver
 */

#include "basics/exceptions.hh"
#include "operator/sparseMatrix.hh"
#include "operator/denseMatrix.hh"
#include "operator/nrlSolver.hh"

using namespace concepts;

int main(int argc, char** argv) {

  try {
    DummySpace<Real> spc1(4);
    std::cout << spc1 << std::endl;
    SparseMatrix<Real> m(spc1, spc1);
    /* consider an example matrix (4X4) with non zero entries at
       [1,1],[1,2],[2,1],[2,2],[2,4],[3,3],[3,4],[4,2],[4,3],[4,4] */
    m(0,0) = 1; m(0,1) = 1; m(1,0) = 1;
    m(1,1) = 1; m(1,3) = 1; m(2,2) = 1;
    m(2,3) = 1; m(3,1) = 1; m(3,2) = 1;
    m(3,3) = 1;
    std::cout << m << " = " << DenseMatrix<Real>(m) << std::endl;
    NRLSolver solver1(m);

    DummySpace<Real> spc(5);
    std::cout << spc << std::endl;
    SparseMatrix<Real> mS(spc, spc);
    mS(0,0) = 2.0; mS(0,1) = mS(1,0) = 0.3; mS(0,3) = mS(3,0) = 0.5;
    mS(1,1) = 3.0;
    mS(2,2) = 7.0; mS(2,4) = mS(4,2) = 2.0;
    mS(3,3) = 8.0; mS(3,4) = mS(4,3) = 6.0;
    mS(4,4) = 7.0;
    std::cout << mS << " = " << DenseMatrix<Real>(mS) << std::endl;

    NRLSolver solver(mS);
    std::cout << solver << std::endl;

    Vector<Real> rhs(spc), sol(spc);
    rhs[0] = 5.14; rhs[1] = 9.9; rhs[2] = 10.3; rhs[3] = 3.8; rhs[4] = 0.7;
    std::cout << "rhs = " << rhs << std::endl;

    solver(rhs, sol);
    std::cout << "sol = " << sol << " should be "
	      << "[1.56618, 3.14338, 2.13892, 2.12927, -2.33621]" << std::endl;
  }
  catch (ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }

  return 0;
}
