/* testsuite for SPOOLES
 */

#include "basics/exceptions.hh"
#include "operator/sparseMatrix.hh"
#include "operator/denseMatrix.hh"
#include "operator/spooles.hh"

using namespace concepts;

int main(int argc, char** argv) {

  try {
    DummySpace<Real> spc(5);
    std::cout << spc << std::endl;
    SparseMatrix<Real> mS(spc, spc);
    mS(0,0) = 2.0; mS(0,1) = mS(1,0) = 0.3; mS(0,3) = mS(3,0) = 0.5;
    mS(1,1) = 3.0;
    mS(2,2) = 7.0; mS(2,4) = mS(4,2) = 2.0;
    mS(3,3) = 8.0; mS(3,4) = mS(4,3) = 6.0;
    mS(4,4) = 7.0;
    std::cout << mS << " = " << DenseMatrix<Real>(mS) << std::endl;

    Spooles solver(mS);
    std::cout << solver << std::endl;

    Vector<Real> rhs(spc), sol(spc);
    rhs[0] = 5.14; rhs[1] = 9.9; rhs[2] = 10.3; rhs[3] = 3.8; rhs[4] = 0.7;
    std::cout << "rhs = " << rhs << std::endl;

    solver(rhs, sol);
    std::cout << "sol = " << sol << " should be "
	      << "[1.56618, 3.14338, 2.13892, 2.12927, -2.33621]" << std::endl;

    SparseMatrix<Real> mU(spc, spc); // same as example in Umfpack Guide p. 10
    mU(0,0) = 2; mU(0,1) = 3;
    mU(1,0) = 3; mU(1,2) = 4; mU(1,4) = 6;
    mU(2,1) = -1; mU(2,2) = -3; mU(2,3) = 2;
    mU(3,2) = 1;
    mU(4,1) = 4; mU(4,2) = 2; mU(4,4) = 1;
    std::cout << mU << " = " << DenseMatrix<Real>(mU) << std::endl;
    Spooles solverU(mU);

    Vector<Real> b(spc);
    b[0] = 8; b[1] = 45; b[2] = -3; b[3] = 3; b[4] = 19;
    std::cout << "rhs = " << rhs << std::endl;
    solverU(b, sol);
    std::cout << "sol = " << sol << " should be "
	      << "[1, 2, 3, 4, 5]" << std::endl;
  }
  catch (ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }

  return 0;
}
