#ifndef transpose_hh
#define transpose_hh

#include "operator/matrix.hh"

namespace concepts {

// *************************************************************** Transpose **

/// The transpose of another matrix
template<class F>
class Transpose : public Matrix<F> {
public:
  /** Constructor.
      @param a The matrix to transpose
  */
  Transpose(Matrix<F>& a) : a_(a) { }
  /// Returns entry with indices \c i and \c j
  virtual F operator()(const uint i, const uint j) const { return a_(j, i); }
  /// Returns and allows access to entry with indices \c i and \c j
  virtual F& operator()(const uint i, const uint j) { return a_(j, i); }
  /// Computes \c fncX = A(\c fncY) where A is this matrix.
  virtual void operator()(const Function<F>& fncY, Function<F>& fncX);
  /// Computes \c fncX = A(\c fncY) where A is this matrix.
  virtual void operator()(const Vector<F>& fncY, Vector<F>& fncX) {
    a_.transpMult(fncY, fncX);
  }
  /// Computes \c fncX = A<sup>T</sup> \c fncY where A is this matrix
  virtual void transpMult(const Vector<F>& fncY, Vector<F>& fncX) {
    a_(fncY, fncX);
  }
  /// Returns the image space
  virtual const Space<F>& spaceX() const { return a_.spaceY(); }
  /// Returns the source space
  virtual const Space<F>& spaceY() const { return a_.spaceX(); }
protected:
  std::ostream& info(std::ostream& os) const;
private:
  /// The matrix to transpose
  Matrix<F>& a_;
};

} // namespace concepts

#endif // transpose_hh
