/* Functions
 * Functions of a in the FE space approximated function
 */

#include "space/function.hh"

namespace concepts {

  template<class F>
  void ScalarElementFunction<F>::operator() (const Element<F>& elm,
					     const Array<F>& coeff,
					     Array<F>& val,
					     const uint *i) const {
    val.resize(1);
    operator()(elm, coeff, val[0], i);
  }

  template<class F>
  void ScalarElementFunction<F>::operator() (const Element<F>& elm,
					     const uint* j, Array<F>& val,
					     const uint* i) const {
    val.resize(1);
    operator()(elm, j, val[0], i);
  }

  template<class F>
  void ScalarElementFunction<F>::operator() (const Element<F>& elm,
					     const Array<F>& coeff,
					     Array<F>& val, const Real p, 
					     const Real t) const {
    val.resize(1);
    operator()(elm, coeff, val[0], p, t);
  }

  template<class F>
  void ScalarElementFunction<F>::operator() (const Element<F>& elm,
					     const Array<F>& coeff,
					     Array<F>& val, const Real2d& p,
					     const Real t) const {
    val.resize(1);
    operator()(elm, coeff, val[0], p, t);
  }

  template<class F>
  void ScalarElementFunction<F>::operator() (const Element<F>& elm,
					     const Array<F>& coeff,
					     Array<F>& val, const Real3d& p,
					     const Real t) const {
    val.resize(1);
    operator()(elm, coeff, val[0], p, t);
  }

  template class ScalarElementFunction<Real>;

  // *************************************************************** Squared **

  template<class F>
  void Squared<F>::operator() (const Element<F>& elm, const Array<F>& coeff,
			       Array<F>& val, const uint *i) const {
    fun_(elm, coeff, val, i);
    square_(val);
  }

  template<class F>
  void Squared<F>::operator() (const Element<F>& elm, const uint* j,
			       Array<F>& val, const uint* i) const {
    fun_(elm, j, val, i);
    square_(val);
  }

  template<class F>
  void Squared<F>::operator() (const Element<F>& elm, const Array<F>& coeff,
			       Array<F>& val, const Real p,
			       const Real t) const {
    fun_(elm, coeff, val, p, t);
    square_(val);
  }

  template<class F>
  void Squared<F>::operator() (const Element<F>& elm, const Array<F>& coeff,
			       Array<F>& val, const Real2d& p,
			       const Real t) const {
    fun_(elm, coeff, val, p, t);
    square_(val);
  }

  template<class F>
  void Squared<F>::operator() (const Element<F>& elm, const Array<F>& coeff,
			       Array<F>& val, const Real3d& p,
			       const Real t) const {
    fun_(elm, coeff, val, p, t);
    square_(val);
  }

  template class Squared<Real>;

}
