/* control structures for high order methods
 */

#include "hpMethod.hh"

namespace concepts {

  // ****************************************************** AdaptiveControlP **

  template<int dim>
  std::ostream& AdaptiveControlP<dim>::info(std::ostream& os) const {
    os << "AdaptiveControlP<" << dim << ">("
       << tag_ << ", idx = " << idx_ << ", p = (";
    for (int i = 0; i < dim; ++i) os << p_[i] << (i == dim-1 ? ")" : ", ");
    return os << ')';
  }

  template class AdaptiveControlP<1>; // 1D
  template class AdaptiveControlP<2>; // 2D
  template class AdaptiveControlP<3>; // 3D

  // ******************************************************* AdaptiveAdjustP **

  template<int dim>
  AdaptiveAdjustP<dim>& AdaptiveAdjustP<dim>::operator=
  (const AdaptiveAdjustP<dim>& a) {
    if (&a != this) {
      std::memcpy(p_, a.p_, dim*sizeof(short));
      level_ = a.level_;
    }
    return *this;
  }

  template<int dim>
  std::ostream& AdaptiveAdjustP<dim>::info(std::ostream& os) const {
    os << "AdaptiveAdjustP(" << level_ << ", p = (";
    for (int i = 0; i < dim; ++i) os << p_[i] << (i == dim-1 ? ")" : ", ");
    return os << ')';
  }

  template class AdaptiveAdjustP<1>;
  template class AdaptiveAdjustP<2>;
  template class AdaptiveAdjustP<3>;

  // *********************************************************** HRefinement **

  template<typename F, int dim>
  void HRefinement<F, dim>::operator() (const Element<F>& elm)
    throw(MissingFeature) {
    const AdaptiveAdjustP<dim> a(1, 0);
    spc_.adjust(elm, a);
  }

  template<typename F, int dim>
  std::ostream& HRefinement<F, dim>::info(std::ostream& os) const {
    return os << "HRefinement()";
  }

  template class HRefinement<Real, 1>;
  template class HRefinement<Real, 2>;
  template class HRefinement<Real, 3>;

  // *********************************************************** PRefinement **

  template<typename F, int dim>
  void PRefinement<F, dim>::operator() (const Element<F>& elm)
    throw(MissingFeature) {
    const AdaptiveAdjustP<dim> a(0, 1);
    spc_.adjust(elm, a);
  }

  template<typename F, int dim>
  std::ostream& PRefinement<F, dim>::info(std::ostream& os) const {
    return os << "PRefinement()";
  }

  template class PRefinement<Real, 1>;
  template class PRefinement<Real, 2>;
  template class PRefinement<Real, 3>;

} // namespace concepts
