/* abstract classes for a space
 */

#include "space.hh"

namespace concepts {

  // ***************************************************************** Space **

  template<class F>
  std::ostream& Space<F>::info(std::ostream& os) const {
    return os << "Space()" << std::endl;
  }

  template class Space<Real>;
  template class Space<Cmplx>;

  // **************************************************** AdaptiveControlTag **

  std::ostream& operator<<(std::ostream& os, const AdaptiveControlTag& c) {
    return os << "AdaptiveControlTag(" << (uint)c.data_ << ": "
	      << (c.isMember() ? "" : "not ") << "member, "
	      << (c.isActive() ? "active" : "passive") << ", "
	      << (c.idxValid() ? "valid" : "invalid") << " index, "
	      << (c.pValid() ? "valid" : "invalid") << " p"
	      << ')';
  }

  // ******************************************************* AdaptiveControl **

  std::ostream& operator<<(std::ostream& os, const AdaptiveControl& c) {
    return os << "AdaptiveControl(" << c.tag_ << ", idx = " << c.idx_ << ')';
  }

  // ******************************************************** AdaptiveAdjust **

  template<uint levelDim>
  std::ostream& operator<<(std::ostream& os,
			   const AdaptiveAdjust<levelDim>& a) {
    return os << "AdaptiveAdjust(" << a.level_ << ')';
  }

  template std::ostream& operator<<(std::ostream&, const AdaptiveAdjust<1>&);
  template std::ostream& operator<<(std::ostream&, const AdaptiveAdjust<2>&);
  template std::ostream& operator<<(std::ostream&, const AdaptiveAdjust<3>&);

  template class AdaptiveAdjust<1>;
  template class AdaptiveAdjust<2>;
  template class AdaptiveAdjust<3>;

} // namespace concepts
