/* testsuite for TMatrix from tmatrix.hh
 */

#include <iostream>
#include "space/tmatrix.hh"
#include "space/element.hh"
#include "basics/exceptions.hh"

using namespace concepts;

int main(int, char**) {

  try {
    ElementMatrix<Real> e(3,3);
    uint k = 1;
    for (uint i = 0; i < e.n(); ++i)
      for (uint j = 0; j < e.m(); ++j)
	e(i, j) = k++;
    std::cout << "e = " << e << std::endl;

    ElementMatrix<Real> f(2,2);
    f.zeros();
    std::cout << "f = " << f << std::endl;

    uint idx[3];
    idx[0] = 0;
    idx[1] = 1;
    idx[2] = 2;
    TIndex<Real> g(3, 3, idx);
    std::cout << "g = " << g << std::endl;

    g(e, f);
    std::cout << "f = " << f << std::endl;

    f(0,0) = 10.0;

    std::cout << "e = " << e << std::endl;
    std::cout << "f = " << f << std::endl << std::endl;

    TColumn<Real> h(3, 0, 0);
    h[0] = 1.0;
    std::cout << "h = " << h << std::endl;

    TColumn<Real> j(3, 0, &h);
    j[1] = 1.0;
    std::cout << "j = " << j << std::endl;

    TMatrix<Real> l(0);
    std::cout << "l = " << l << std::endl;
    l.append(&j);
    std::cout << "l = " << l << std::endl;

    l(e,f);
    std::cout << "f = " << f << std::endl;
    f.transpose();
    ElementMatrix<Real> m(1,1);
    l(f,m);
    m.transpose();
    std::cout << "m = " << m << std::endl;

    uint n[2] = { 2, 3 };
    TColumnTensor<Real, 2> o(n, 3);
    TColumnTensor<Real, 2> p(n, 4, &o);
    n[0] = 1; n[1] = 1;
    o[n] = 1.0;
    p[n] = 2.0;
    std::cout << "o = " << o << std::endl;
    std::cout << "p = " << p << std::endl;

    uint q[3] = { 2, 3, 1 };
    TColumnTensor<Real, 3> r(q, 5);
    q[0] = 1; q[1] = 1; q[2] = 0;
    r[q] = 1.0;
    q[0] = 0; q[1] = 1; q[2] = 0;
    r[q] = 3.0;
    std::cout << "r = " << r << std::endl;
    std::cout << "r = " << (TColumn<Real>)r << std::endl;

    TMatrix<Real> s(0);
    std::cout << "s = " << s << std::endl;
    s.append(&p);
    std::cout << "s = " << s << std::endl;
    s.append(&r);
    std::cout << "s = " << s << std::endl;

    TColumn<Real>* t = r.clone();
    std::cout << "t = " << *t << std::endl;
    delete t;
    t = h.clone();
    std::cout << "t = " << *t << std::endl;
    delete t;

    TMatrixBase<Real>* v = new TMatrix<Real>(s);
    std::cout << "v = " << *v << std::endl;
    delete v;
    v = new TIndex<Real>(3, 3, idx);
    std::cout << "v = " << *v << std::endl;
    delete v;

    Array<uint> aa(2, 0);
    TIndex<Real> ab(2, 2, (uint*) aa);
    std::cout << "ab = " << ab << std::endl;
    TColumn<bool> ac(2, 1);
    std::cout << "ac = " << ac << std::endl;
    ab.usedIdx(ac);
    std::cout << "ac = " << ac << std::endl;

    q[0] = 2; q[1] = 3; q[2] = 1;
    TColumnTensor<bool, 3> w(q, 5);
    q[0] = 1; q[1] = 1; q[2] = 0;
    w[q] = true;
    std::cout << "w = " << w << std::endl;
    s.usedIdx(w);
    std::cout << "w = " << w << std::endl;
    std::cout << "w = " << (TColumn<bool>)w << std::endl;
    const uint* an = w.n();
    uint i[3];
    for (i[2] = 0; i[2] < an[2]; ++i[2]) {
      for (i[1] = 0; i[1] < an[1]; ++i[1]) {
	for (i[0] = 0; i[0] < an[0]; ++i[0]) {
	  std::cout << "(" << i[0] << ", " << i[1] << ", " << i[2] << ") "
		    << "do" << (w[i] ? " " : " not do ") << "it" << std::endl;
	}
      }
    }
  }
  catch(ExceptionBase& e) {
    std::cout << e << std::endl;
    return 1;
  }

  return 0;
}
