/*----------------------------------------------------------------------------
Sparse QR solver for linear systems of equations or least squares problems
Copyright (C) 1996, Thomas H. Robey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Inquiries can be directed to trobey@arc.unm.edu or

Thomas H. Robey
925 Madison NE
Albuquerque, NM  87110
USA
----------------------------------------------------------------------------*/

#ifndef fill_hh
#define fill_hh

#include <cstddef>

namespace sparseqr {

  /** @author Thomas H. Robey, 1996, 1997, 1998
      @see <a href="http://www.ahpcc.unm.edu/~trobey/#SPARSEQR">Homepage of SparseQR</a><br>
      Thomas H. Robey and Deborah L. Sulsky, <i>Row Ordering for Sparse QR Decomposition,</i> SIAM J. Matrix Anal. and Appl., Vol. 15, No. 4, pp. 1208-1225, October 1994.
  */
  class Fill {
  public:
    int CalcFill(Smatrix *,Smatrix *);
    Fill(Smatrix *,int);
    ~Fill();
    void IndexSort(QR *,int *,int);
    void Init(QR *,int);
    void Merge(QR *qr);
    void PivotFill(QR *qr) {
      for (int i=0;i<nlist2;i++)
	pairfill[sortindex2[i]]=CalcFill(qr->srow[row1[sortindex2[i]]],
					 qr->srow[row2[sortindex2[i]]]);
    };
    void Update(int,int);
    int friend QR::sparseqr();
  private:
    int nlist1,nlist2;
    int *sortindex1,*sortindex2,*row1,*row2,*pairfill,*rindex;
  };

}

#endif // fill_hh
