/*----------------------------------------------------------------------------
Sparse QR solver for linear systems of equations or least squares problems
Copyright (C) 1996, Thomas H. Robey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Inquiries can be directed to trobey@arc.unm.edu or

Thomas H. Robey
925 Madison NE
Albuquerque, NM  87110
USA
----------------------------------------------------------------------------*/

#include "sparseqr.hh"
#include "mindegree.hh"
#include <iostream>

namespace sparseqr {

  MinDegree::MinDegree(QR *qr,int nc) {
    degree=new int[nc];
    // std::cout << "allocated degree of size " << nc << std::endl;
    assert(degree!=NULL);
    for (int i=0;i<nc;i++)
      degree[i]=-1;
    work=new int[nc];
    for (int i=0;i<nc;i++)
      work[i]=-1;
    for (int i=0;i<nc;i++) {
      for (Smatrix *cptr=qr->scol[i];cptr!=NULL;cptr=cptr->down) {
	for (Smatrix *rptr=qr->srow[cptr->row];rptr!=NULL;rptr=rptr->right) {
	  if (work[rptr->col]!=i) {
	    degree[i]++;
	    work[rptr->col]=i;
	  }
	}
      }
    }
    delete[] work;
  };

  int MinDegree::MinSearch(QR *qr,int c,int nnzc,int *md) {
    int k=c;
    *md=degree[qr->pcol[k]];
    for (int i=c+1;i<nnzc;i++) {
      if (degree[qr->pcol[i]]<*md || (degree[qr->pcol[i]]==*md && 
				      qr->pcol[i]<qr->pcol[k])) {
	k=i;
	*md=degree[qr->pcol[i]];
      }
    }
    return(k);
  };

  void MinDegree::Track(QR *qr,int md,int c,int nnzc) {
    if (md==0)
      return;
    list=new int[md];
#if DEBUG
    for (int i = 0; i < md; ++i) list[i] = -1;
#endif
    assert(list!=NULL);
    work=new int[qr->nc];
    assert(work!=NULL);
    for (int i=c+1;i<nnzc;i++)
      work[qr->pcol[i]]=-1;
    int i=0;
    for (Smatrix *cptr=qr->scol[qr->pcol[c]];i<md && cptr!=NULL;
	 cptr=cptr->down) {
      for (Smatrix *rptr=qr->srow[cptr->row];rptr!=NULL;rptr=rptr->right) {
	if (rptr->col!=qr->pcol[c] && work[rptr->col]!=rptr->col) {
	  list[i]=rptr->col;
	  i++;
	  work[rptr->col]=rptr->col;
	}
      }
    }
    delete[] work;
  };

  void MinDegree::Untrack(QR *qr,int p,int md,int c,int nnzc) {
    if (md==0)
      return;
    work=new int[qr->nc];
    assert(work!=NULL);
    for (int i=c;i<nnzc;i++)
      work[qr->pcol[i]]=-1;
    for (int i=0;i<md;i++) {
      Smatrix *cptr=NULL;
      degree[list[i]]=-1;
      for (Smatrix *sptr=qr->scol[list[i]];sptr!=NULL;sptr=sptr->down) {
	if (sptr->row==p) {
	  if (cptr==NULL)
	    qr->scol[list[i]]=sptr->down;
	  else
	    cptr->down=sptr->down;
	}
	else {
	  for (Smatrix *rptr=qr->srow[sptr->row];rptr!=NULL;rptr=rptr->right) {
	    if (work[rptr->col]!=list[i]) {
	      degree[list[i]]++;
	      work[rptr->col]=list[i];
	    }
	  }
	}
	cptr=sptr;
      }
    }
    delete[] work;
    delete[] list;
  };

}
