// euler strategy

#include "euler.hh"
#include "operator/compositions.hh"

// debugging
#include "basics/debug.hh"

#define Euler_D 0

namespace timestepping {

  // ************************************************************* Euler **

  Euler::Euler(concepts::SolverFabric<Real>& fabric,
	       concepts::Operator<Real>& D1,
	       concepts::Operator<Real>& D0,
	       TimeVector& trhs, 
	       const concepts::Vector<Real>& Y0,
	       Real dt):
    TimeStepStrategy(fabric, &D1, Y0, Y0, dt),
    D1_(D1), D0_(D0), trhs_(trhs), Yn1_(Y0) {}

  Euler::~Euler() {}

  void Euler::next() {
    Yn1_ = sol_;
    Yn1_.time(t_);
 
    TimeVector temp1(Yn1_);
    
    // build dt*L_h(Yn1_)    
    D0_(Yn1_, temp1);
    trhs_.time(t_);
    temp1 -= trhs_;
    temp1 *= -dt_;
    DEBUGL(Euler_D, "-D_0 + g(t)  = " << temp1 << std::endl);
    (*solver_)(temp1, rhs_);
    
    sol_ += rhs_;
    t_ += dt_;
  }

  std::ostream& Euler::info(std::ostream& os) const {
    return os << "Euler(dt = " << dt_ << ')';
  }

  // ***************************************************** LimitingEuler **

  LimitingEuler::LimitingEuler(concepts::SolverFabric<Real>& fabric,
			       concepts::Operator<Real>& D1,
			       concepts::Operator<Real>& D0,
			       concepts::Operator<Real>& limiter,
			       TimeVector& trhs,
			       const concepts::Vector<Real>& Y0,
			       Real dt) :
    TimeStepStrategy(fabric, &D1, Y0, Y0, dt),
    D1_(D1), D0_(D0), limiter_(limiter), trhs_(trhs), Yn1_(Y0) {}

  LimitingEuler::~LimitingEuler() {}

  void LimitingEuler::next() {
    Yn1_ = sol_;
    Yn1_.time(t_);

    TimeVector temp1(Yn1_);
    
    limiter_(Yn1_, sol_);

    // build dt*L_h(Yn1_)   
    D0_(Yn1_, temp1);
    trhs_.time(t_);
    temp1 -= trhs_;
    temp1 *= -dt_;
    DEBUGL(Euler_D, "-D_0 + g(t)  = " << temp1 << std::endl);
    (*solver_)(temp1, rhs_);

    sol_ += rhs_;
    t_ += dt_;
  }

  std::ostream& LimitingEuler::info(std::ostream& os) const {
    return os << "LimitingEuler(dt = " << dt_ << ')';
  }
}
