// newmark strategy

#include "newmark.hh"
#include "vectors.hh"
#include "operator/compositions.hh"

// debugging
#include "basics/debug.hh"

#define Newmark_D 0

namespace timestepping {

  // *************************************************************** Newmark **

  Newmark::Newmark(concepts::SolverFabric<Real>& fabric,
		   concepts::Operator<Real>& D2,
		   concepts::Operator<Real>& D0,
		   timestepping::TimeVector& trhs, 
		   const concepts::Vector<Real>& Y0,
		   const concepts::Vector<Real>& Z0,
		   Real dt, Real beta, Real gamma) :
    TimeStepStrategy(fabric, new concepts::LiCo<Real>(D2, D0, 1.0, beta*dt*dt),
		     Y0, Z0, dt),
    D2_(D2), D0_(D0), trhs_(trhs), Yn1_(Y0), Yn2_(Y0),
    beta_(beta), gamma_(gamma), lhs_(liCo_) {}

  void Newmark::next() {
    Yn2_ = Yn1_;
    Yn1_ = sol_;
    t_ += dt_;

    if(t_ != dt_) {
      concepts::Vector<Real> temp1(Yn1_);
      trhs_.time(t_);
      concepts::Vector<Real> temp2(trhs_);

      temp1 *= 2;
      temp1 -= Yn2_;
      D2_(temp1, rhs_);
      DEBUGL(Newmark_D, "1. Summand = " << rhs_ << std::endl);

      temp2 *= beta_;
      D0_(Yn1_, temp1);
      temp1 *= -1;
      trhs_.time(t_-dt_);
      temp1 += trhs_;
      temp1 *= 0.5-2*beta_+gamma_;
      temp2 += temp1;
      D0_(Yn2_, temp1);
      temp1 *= -1;
      trhs_.time(t_-2*dt_);
      temp1 += trhs_;
      temp1 *= 0.5+beta_-gamma_;
      temp2 += temp1;
      temp2 *= dt_*dt_;
      DEBUGL(Newmark_D, "2. Summand = " << temp1 << std::endl);
      rhs_ += temp2; 
    }
    else {                                   // 1. step is different
      concepts::Vector<Real> temp1(rhs_);
      trhs_.time(dt_);
      concepts::Vector<Real> temp2(trhs_);

      temp1 *= dt_;
      temp1 += Yn1_;
      D2_(temp1, rhs_);
      DEBUGL(Newmark_D, "1. Summand = " << rhs_ << std::endl);

      D0_(Yn1_, temp1);
      temp1 *= -1;
      trhs_.time(0);
      temp1 += trhs_;
      temp1 *= (0.5-beta_);

      temp2 *= beta_;
      temp1 += temp2;
      temp1 *= dt_*dt_;
      DEBUGL(Newmark_D, "2. Summand = " << temp1 << std::endl);
      rhs_  += temp1;
    }
    (*solver_)(rhs_, sol_);
  }

  std::ostream& Newmark::info(std::ostream& os) const {
    return os << "Newmark(beta = " << beta_ << ", gamma = " << gamma_
	      << ", dt = " << dt_ << ')';
  }

}
