// runge kutta 2 strategy

#include "rungekutta2.hh"
#include "vectors.hh"
#include "operator/compositions.hh"

// debugging
#include "basics/debug.hh"

#define RungeKutta2_D 0

namespace timestepping {

  // *********************************************************** RungeKutta2 **

  RungeKutta2::RungeKutta2(concepts::SolverFabric<Real>& fabric,
			   concepts::Operator<Real>& D1,
			   concepts::Operator<Real>& D0,
			   timestepping::TimeVector& trhs, 
			   const concepts::Vector<Real>& Y0,
			   Real dt, Real alpha) :
    TimeStepStrategy(fabric, &D1, Y0, Y0, dt),
    D1_(D1), D0_(D0), trhs_(trhs), Yn1_(Y0), alpha_(alpha) {}

  RungeKutta2::~RungeKutta2() {}

  void RungeKutta2::next() {
    Yn1_ = sol_;
    TimeVector temp1(sol_);
    TimeVector temp2(sol_);

    temp2.time(t_);
    D0_(temp2, temp1);
    trhs_.time(t_);
    temp1 -= trhs_;
    (*solver_)(temp1, temp2);
    temp1 = temp2;
    temp1 *= -dt_ * (1.0-1.0/(2.0*alpha_));
    DEBUGL(RungeKutta2_D, "1. Summand  = " << temp1 << std::endl);
    sol_ += temp1;

    temp2 *= -alpha_ * dt_;
    temp2 += Yn1_;
    temp2.time(t_ + alpha_ * dt_);
    D0_(temp2, temp1);
    trhs_.time(t_ + alpha_ * dt_);
    temp1 -= trhs_;
    (*solver_)(temp1, temp2);
    temp2 *= -dt_ * (1.0/(2.0*alpha_));
    DEBUGL(RungeKutta2_D, "2. Summand = " << temp2 << std::endl);
    sol_ += temp2;

    t_ += dt_;
  }

  std::ostream& RungeKutta2::info(std::ostream& os) const {
    return os << "RungeKutta2(alpha = " << alpha_ << ", dt = " << dt_ << ')';
  }

}
