// theta strategy

#include "theta.hh"
#include "vectors.hh"
#include "operator/compositions.hh"

// debugging
#include "basics/debug.hh"

#define Theta_D 0

namespace timestepping {

  // *************************************************************** Theta **

  Theta::Theta(concepts::SolverFabric<Real>& fabric,
	       concepts::Operator<Real>& D1,
	       concepts::Operator<Real>& D0,
	       timestepping::TimeVector& trhs, 
	       const concepts::Vector<Real>& Y0,
	       Real dt, Real theta) :
    TimeStepStrategy(fabric, new concepts::LiCo<Real>(D1, D0, 1.0, theta*dt),
		     Y0, Y0, dt),
    D1_(D1), D0_(D0), trhs_(trhs), Yn1_(Y0), theta_(theta), lhs_(liCo_) {}

  Theta::~Theta() {}

  void Theta::next() {
    Yn1_ = sol_;
    DEBUGL(Theta_D, "Yn1 = " << Yn1_ << std::endl);
    t_ += dt_;
    
    concepts::Vector<Real> temp1(Yn1_);
    trhs_.time(t_-dt_);
    concepts::Vector<Real> temp2(trhs_);

    D1_(Yn1_, rhs_);
    DEBUGL(Theta_D, "1. Summand = " << rhs_ << std::endl);
    
    D0_(Yn1_, temp1);
    temp1 -= temp2;
    temp1 *= (theta_ - 1.0);
    trhs_.time(t_);
    temp2 = trhs_;
    temp2 *= theta_;
    temp1 += temp2;
    temp1 *= dt_;
    DEBUGL(Theta_D, "2. Summand = " << temp1 << std::endl);
    rhs_ += temp1;
    (*solver_)(rhs_, sol_);
  }

  std::ostream& Theta::info(std::ostream& os) const {
    return os << "Theta(theta = " << theta_
	      <<", dt = " << dt_ << ')';
  }

}
