#ifndef formulaLiCo_hh
#define formulaLiCo_hh

#include "toolbox/formula.hh"

namespace concepts {

  /** Linear combination of two formulae.
      Calculates cA * frmA(x,t) + cB * frmB(x,t).
      @author Christoph Winkelmann, 2003
  */
  class FormulaLiCo : public Formula {
  public:
    /** Constructor.
        @param frmA formula A
        @param frmB formula B
        @param cA coefficient of \c frmA
        @param cB coefficient of \c frmB
    */
    FormulaLiCo(const Formula& frmA, const Formula& frmB, Real cA, Real cB)
      : frmA_(frmA), frmB_(frmB), cA_(cA), cB_(cB) { }
    /// Return a clone of this linear combination
    virtual Formula * clone () const {
      return new FormulaLiCo(frmA_, frmB_, cA_, cB_);
    }

    /// Application operator for one dimension
    virtual Real operator() (const Real p, const Real t=0.0) const {
      return cA_*frmA_(p, t) + cB_*frmB_(p, t);
    }
    /// Application operator for two dimensions
    virtual Real operator() (const Real2d &p, const Real t=0.0) const {
      return cA_*frmA_(p, t) + cB_*frmB_(p, t);
    }
    /// Application operator for three dimensions
    virtual Real operator() (const Real3d &p, const Real t=0.0) const {
      return cA_*frmA_(p, t) + cB_*frmB_(p, t);
    }
  private:
    const Formula& frmA_;
    const Formula& frmB_;
    const Real cA_;
    const Real cB_;
  };

} // namespace concepts

#endif //formulaLiCo_hh
