/* C declarations */
%{
#include <cmath>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <fstream>
#include "toolbox/inputOutput.hh"

extern "C" int yylex(...);

/* Called by yyparse on error */
int yyerror (char* s) {
  std::cout << s << std::endl;

  return 0;
}

char* YYarray = 0;

#define YYPARSE_PARAM param

using namespace std;
%}

/* bison declarations */
%union {
  double value;
  int ganz;
  const char* name;
}

%token <name> ANYTHING
%token <value> NUM
%token <ganz> GANZ
%token <ganz> TRUEFALSE
%token <name> NAME

%token REAL
%token STRING
%token INT
%token BOOL
%token ARRAY

%pure_parser

%%

/* grammar rules */
input:    /* empty */
        | input line
;

line:     '\n'
        | REAL NAME NUM '\n' { 
	  ((concepts::InOutParameters*)param)->addDouble($2, $3); }
        | STRING NAME ANYTHING '\n' { 
	  ((concepts::InOutParameters*)param)->addString($2, $3); }
        | INT NAME GANZ '\n' { 
	  ((concepts::InOutParameters*)param)->addInt($2, $3); }
        | BOOL NAME TRUEFALSE '\n' { 
	  ((concepts::InOutParameters*)param)->addBool($2, $3); }

        | ARRAY STRING NAME '{' '\n' { 
	  YYarray = new char[std::strlen($3)+1];
	  std::strncpy(YYarray, $3, std::strlen($3));
	  YYarray[std::strlen($3)] = '\0'; 
	  ((concepts::InOutParameters*)param)->addArrayString($3); }
        | ARRAY REAL NAME '{' '\n' { 
	  YYarray = new char[std::strlen($3)+1];
	  std::strncpy(YYarray, $3, std::strlen($3));
	  YYarray[std::strlen($3)] = '\0'; 
	  ((concepts::InOutParameters*)param)->addArrayDouble($3); }
        | ARRAY INT NAME '{' '\n' { 
	  YYarray = new char[std::strlen($3)+1];
	  std::strncpy(YYarray, $3, std::strlen($3));
	  YYarray[std::strlen($3)] = '\0'; 
	  ((concepts::InOutParameters*)param)->addArrayInt($3); }
        | ARRAY BOOL NAME '{' '\n' { 
	  YYarray = new char[std::strlen($3)+1];
	  std::strncpy(YYarray, $3, std::strlen($3));
	  YYarray[std::strlen($3)] = '\0'; 
	  ((concepts::InOutParameters*)param)->addArrayBool($3); }

        | GANZ ANYTHING '\n' { 
	  ((concepts::InOutParameters*)param)->addArrayString(YYarray, $1, $2); }
        | GANZ NUM '\n' { 
	  ((concepts::InOutParameters*)param)->addArrayDouble(YYarray, $1, $2); }
        | GANZ GANZ '\n' { 
	  ((concepts::InOutParameters*)param)->addArrayInt(YYarray, $1, $2); }
        | GANZ TRUEFALSE '\n' { 
	  ((concepts::InOutParameters*)param)->addArrayBool(YYarray, $1, $2); }

        | '}' '\n' { delete[] YYarray; }

;
%%
