/* pool
 * blockwise memory allocation
 */

#ifndef pool_hh
#define pool_hh

// debugging
#include "basics/debug.hh"

#define PoolConstr_D 0

namespace concepts {

  // ****************************************************************** Pool **
 
  /** Pool for blockwise memory allocation.
      A chunk of memory is allocated and returned piecewise. If all
      space is used up, new space is allocated.
  */
  template <class T> 
  class Pool {
    union link {
      void*  lnk;
      double align;
    };

    const unsigned  blkSz;
    void*           blk;

    /// Stores the number of free entries
    unsigned        free;

    T*              next;

    Pool(Pool&);
    void operator=(Pool&);
  public:
    /** Constructor.
     */
    Pool(unsigned sz) : blkSz(sz), blk(0), free(0), next(0) {
      DEBUGL(PoolConstr_D, "done.");
    }
    ~Pool();

    /** Returns a pointer to a available location. If needed, a new chunk
	is allocated.
    */
    T* alloc();
    float memory() const;
  };

  template <class T> 
  Pool<T>::~Pool() {
    while (blk) {
      char* b = (char*)blk; blk = ((link*) blk)->lnk;
      delete[] b;
    }
  }

  template <class T> 
  T* Pool<T>::alloc() {
    if (!free) { 
      void* b = new char[blkSz * sizeof(T) + sizeof(link)];
      ((link*) b)->lnk = blk; blk = b;
      next = (T*)((link*) blk + 1); free = blkSz;
    }   
    free--; return next++;
  }

  template <class T> 
  float Pool<T>::memory() const {
    float mem = 0.0;
    float blkmem = blkSz * sizeof(T) + sizeof(link);
    for(void* b = blk; b != NULL; b = ((link*)b)->lnk) mem += blkmem;
    return sizeof(Pool) + mem;
  }

} // namespace concepts

#endif // pool_hh
