/* resource monitor
 */

#include <iomanip>
#include <unistd.h>
#include "resourceMonitor.hh"

// debugging
#include "basics/debug.hh"

namespace concepts {

  // ******************************************************* ResourceMonitor **

  std::ostream& ResourceMonitor::sample(std::ostream& os) {
    timeval* utime = &ru_.ru_utime;
    timeval* stime = &ru_.ru_stime;

    float stamp = (utime->tv_sec * 1.0e6 + utime->tv_usec + 
		   stime->tv_sec * 1.0e6 + stime->tv_usec) / 1.0e6;

    getrusage(RUSAGE_SELF, &ru_);

    float ut = (utime->tv_sec * 1.0e6 + utime->tv_usec) / 1.0e6;
    float st = (stime->tv_sec * 1.0e6 + stime->tv_usec) / 1.0e6;

    float res = ut+st-stamp;

    os << ((res < 0.01) ? 0 : res) << "s [utime = " << ut << "s, stime = "
       << st;
#ifdef __SUNPRO_CC
    os << ']';
#else
    os << "s, minflt = " << ru_.ru_minflt << ", majflt = " << ru_.ru_majflt;
    os << ", maxrss = " << (float) ru_.ru_maxrss * getpagesize()/1024.0 
       << "KB]";  
#endif

    return os;
  }

  float ResourceMonitor::utime() {
    timeval* utime = &ru_.ru_utime;
    timeval* stime = &ru_.ru_stime;

    float stamp = (utime->tv_sec * 1.0e6 + utime->tv_usec + 
		   stime->tv_sec * 1.0e6 + stime->tv_usec) / 1.0e6;

    getrusage(RUSAGE_SELF, &ru_);

    float ut = (utime->tv_sec * 1.0e6 + utime->tv_usec) / 1.0e6;
    float st = (stime->tv_sec * 1.0e6 + stime->tv_usec) / 1.0e6;
    float res = ut+st-stamp;
    return (res < 0.01) ? 0 : res;
  }

}
