#ifndef ElementPairListIdentic_hh
#define ElementPairListIdentic_hh

#include "vectorial.hh"
#include "space/elementPairs.hh"

namespace vectorial {

  /** Element pair list for two vectorial spaces V1 and V2. V1 must consist of
      identic scalar spaces S1 in each vectorial dimension, V2 must consist of
      identic scalar spaces S2 in each vectorial dimension. The respective
      scalar element pair is added d1*d2 times to the vectorial element pair,
      where d1 is the vectorial dimension of V1 and d2 is the vectorial
      dimension of V2. The scalar element pairs are needed in the scalar
      bilinear forms building the vectorial bilinear form. In the general case
      of d1*d2 different scalar spaces, the vectorial element pair would have
      to know the relations of d1 different elements from S1 to d2 different
      elements on the right, i. e. d1*d2 different element pairs. For the sake
      of generality, d1*d2 scalar element pairs are added also in the case of
      identic scalar spaces.
      @author Christoph Winkelmann, 2003
  */
  template<class F>
  class ElementPairListIdentic : public concepts::ElementPairList<F> {
  public:
    /** Constructor. Scans \c sElemPairs and adds to the ElementPairList
        represented by this class one vectorial::ElementPair for each scalar
        ElementPair in sElemPairs. The vectorial::Elements needed for
        construction of the vectorial::ElementPair are found in \c vSpace1 and
        \c vSpace2.
    */
    ElementPairListIdentic(const concepts::ElementPairList<F>& sElemPairs,
                           const Space<F>& vSpace1, const Space<F>& vSpace2) {
      constructor_(sElemPairs, vSpace1, vSpace2);
    }
    /** Constructor for two identic vectorial spaces. Scans \c sElemPairs and
        adds to the ElementPairList represented by this class one
        vectorial::ElementPair for each scalar ElementPair in sElemPairs. Both
        vectorial::Elements needed for construction of the
        vectorial::ElementPair are found in \c vSpace.
    */
    ElementPairListIdentic(const concepts::ElementPairList<F>& sElemPairs,
                           const Space<F>& vSpace) {
      constructor_(sElemPairs, vSpace, vSpace);
    }
  private:
    void constructor_(const concepts::ElementPairList<F>& sElemPairs,
                      const Space<F>& vSpace1,
                      const Space<F>& vSpace2);
  };

} // namespace vectorial

#endif // ElementPairListIdentic_hh
