// Helper class for LinearForm: assembling of components

#ifndef linearFormHelper_hh
#define linearFormHelper_hh

#include "vectorial.hh"
#include "function/linearForm.hh"

namespace vectorial {

  // ****************************************************** LinearFormHelper **

  /** Helper class for building a linear form in \c vectorial.
      @author Philipp Frauenfelder, 2003.
  */
  class LinearFormHelper {
  public:
    /** Assembles the scalar contributions of the components into the
	vector valued element load vector. This is done by looping over
	all non-zero entries in \c lfs and calling it with <tt>elm</tt>.
	The result is written to <tt>em</tt>.

	This method is used by vectorial::LinearForm and
	vectorial::TimeLinearForm.
	@param elm Vector valued element to evaluate the linear form on
	@param em Element load vector (result)
	@param lfs List of linear forms
	@param vdim Number of linear forms in \c lfs
     */
    static void assembleComponents(const Element<Real>& elm,
                                   concepts::ElementMatrix<Real>& em,
				   concepts::LinearForm<Real>** const lfs,
				   const uint vdim);
  };

} // namespace vectorial

#endif // linearFormHelper_hh
