#ifndef vectorialTlf_hh
#define vectorialTlf_hh

#include "vectorial/vectorial.hh"
#include "timestepping/vectors.hh"

// debugging
#include "basics/debug.hh"

#define TimeLinearFormConstr_D 0

namespace vectorial {

  using concepts::Real;

  /** Vector valued time dependent linear form. Consists of \c vdim
      timestepping::TimeLinearForms and sets the time of all of them when
      the time is set.
      @author Christoph Winkelmann, 2003
  */
  class TimeLinearForm : public Vectorial<timestepping::TimeLinearForm >,
                         public timestepping::TimeLinearForm {
  public:
    TimeLinearForm(const uint vdim, const uint arrayWidth=0) : 
      Vectorial<timestepping::TimeLinearForm >(vdim,vdim) {
        DEBUGL(TimeLinearFormConstr_D, "done.");
      }
    virtual ~TimeLinearForm();
    
    virtual void put(timestepping::TimeLinearForm &lf,
                     const int i, const int j=0);
    virtual void operator ()(const concepts::Element<Real>& elm,
                             concepts::ElementMatrix<Real>& em);
    /// set the time of all scalar linear forms
    virtual void time(Real time);
  protected:
    virtual std::ostream& info(std::ostream& os) const;
  };
  
} // namespace vectorial

#endif // vectorialTlf_hh
